
"""
test_GeneratorSE.py

Created by Latha Sethuraman on 2017-03-09.
Copyright (c) NREL. All rights reserved.
"""

import unittest
import numpy as np



# from drivese.drive_smooth import YawSystemSmooth, BedplateSmooth
import generatorse.PMSG_arms, generatorse.PMSG_disc, generatorse.DFIG,generatorse.SCIG,generatorse.EESG
from generatorse.PMSG_arms import PMSG_arms_Opt
from generatorse.EESG import EESG_Opt
from generatorse.PMSG_disc import PMSG_disc_Opt
from generatorse.DFIG import DFIG_Opt
from generatorse.SCIG import SCIG_Opt


class Test_PMSG_arms(unittest.TestCase):

    def setUp(self):

        self.PMSG_arms = PMSG_arms_Opt('CONMINdriver','PMSG_Cost.Costs',0)
 
        
        # Initial design variables for a DD PMSG designed for a 5MW turbine
        self.PMSG_arms.Eta_target = 93
        self.PMSG_arms.P_rated=5.0e6
        self.PMSG_arms.T_rated=4.143289e6
        self.PMSG_arms.N=12.1
        self.PMSG_arms.PMSG_r_s= 3.26
        self.PMSG_arms.PMSG_l_s= 1.6
        self.PMSG_arms.PMSG_h_s = 0.07
        self.PMSG_arms.PMSG_tau_p = 0.08
        self.PMSG_arms.PMSG_h_m = 0.009
        self.PMSG_arms.PMSG_h_ys = 0.075
        self.PMSG_arms.PMSG_h_yr = 0.075
        self.PMSG_arms.PMSG_n_s = 5
        self.PMSG_arms.PMSG_b_st = 0.480
        self.PMSG_arms.PMSG_n_r =5
        self.PMSG_arms.PMSG_b_r = 0.530
        self.PMSG_arms.PMSG_d_r = 0.7
        self.PMSG_arms.PMSG_d_s= 0.35
        self.PMSG_arms.PMSG_t_wr =0.06
        self.PMSG_arms.PMSG_t_ws =0.06
        self.PMSG_arms.PMSG_R_o =0.43
        
        # Specific costs
        self.PMSG_arms.C_Cu   =4.786
        self.PMSG_arms.C_Fe	= 0.556
        self.PMSG_arms.C_Fes =0.50139
        self.PMSG_arms.C_PM  =95
        
        #Material properties
        self.PMSG_arms.rho_Fe = 7700                 #Steel density
        self.PMSG_arms.rho_Fes = 7850                 #Steel density
        self.PMSG_arms.rho_Copper =8900                  # Kg/m3 copper density
        self.PMSG_arms.rho_PM =7450
        

       
    def test_functionality(self):
        
        self.PMSG_arms.run()
        
        self.assertEqual(round(self.PMSG_arms.Mass/1000,2), 101.05)


class Test_PMSG_disc(unittest.TestCase):
	
	def setUp(self):
		
				self.PMSG_disc=PMSG_disc_Opt('CONMINdriver','PMSG_Cost.Costs',0)
				
				
				
				# Initial design variables for a DD PMSG designed for a 1.5MW turbine
				self.PMSG_disc.Eta_target = 93
				self.PMSG_disc.P_rated=5.0e6
				self.PMSG_disc.T_rated=4143289.841
				self.PMSG_disc.N=12.1
				self.PMSG_disc.PMSG_r_s=3.49
				self.PMSG_disc.PMSG_l_s= 1.5
				self.PMSG_disc.PMSG_h_s = 0.06
				self.PMSG_disc.PMSG_tau_p = 0.07
				self.PMSG_disc.PMSG_h_m = 0.0105
				self.PMSG_disc.PMSG_h_ys = 0.085
				self.PMSG_disc.PMSG_h_yr = 0.055
				self.PMSG_disc.PMSG_n_s = 5
				self.PMSG_disc.PMSG_b_st = 0.46
				self.PMSG_disc.PMSG_t_d = 0.105
				self.PMSG_disc.PMSG_d_s= 0.350
				self.PMSG_disc.PMSG_t_ws =0.15
				
				self.PMSG_disc.PMSG_R_o =0.43
				
				# Specific costs
				self.PMSG_disc.C_Cu   =4.786
				self.PMSG_disc.C_Fe	= 0.556
				self.PMSG_disc.C_Fes =0.50139
				self.PMSG_disc.C_PM  =95
				
				#Material properties
				self.PMSG_disc.rho_Fe = 7700                 #Steel density
				self.PMSG_disc.rho_Fes = 7850                 #Steel density
				self.PMSG_disc.rho_Copper =8900                  # Kg/m3 copper density
				self.PMSG_disc.rho_PM =7450
				
				def test_functionality(self):
					
					self.PMSG_disc.run()
					self.assertEqual(round(self.PMSG_disc.Mass/1000,1), 122.5)

class Test_EESG(unittest.TestCase):

    def setUp(self):

        self.EESG = EESG_Opt('CONMINdriver','EESG_Cost.Costs',0)
        
        # Initial design variables for a DD EESG designed for a 5MW turbine
        self.EESG.Eta_target= 93
        self.EESG.P_rated=5.0e6
        self.EESG.T_rated=4.143289e6
        self.EESG.N_rated=12.1
        self.EESG.EESG_r_s=3.2
        self.EESG.EESG_l_s= 1.4
        self.EESG.EESG_h_s = 0.060
        self.EESG.EESG_tau_p = 0.17
        self.EESG.EESG_I_f = 69
        self.EESG.EESG_N_f = 100
        self.EESG.EESG_h_ys = 0.130
        self.EESG.EESG_h_yr = 0.12
        self.EESG.EESG_n_s = 5
        self.EESG.EESG_b_st = 0.47
        self.EESG.EESG_n_r =5
        self.EESG.EESG_b_r = 0.48
        self.EESG.EESG_d_r = 0.51
        self.EESG.EESG_d_s= 0.4
        self.EESG.EESG_t_wr =0.14
        self.EESG.EESG_t_ws =0.07
        self.EESG.EESG_R_o =0.43
        
        # Costs
        self.EESG.C_Cu   =4.786
        self.EESG.C_Fe	= 0.556
        self.EESG.C_Fes =0.50139
        
        #Material properties
        
        self.EESG.rho_Fe = 7700                 #Steel density
        self.EESG.rho_Fes = 7850                 #Steel density
        
        self.EESG.rho_Copper =8900                  # Kg/m3 copper density

	
	
    def test_functionality(self):
        
        self.EESG.run()
        
        self.assertEqual(round(self.EESG.Mass/1000,1), 147.9)


class Test_SCIG(unittest.TestCase):

    def setUp(self):

        self.SCIG = SCIG_Opt('CONMINdriver','SCIG_Cost.Costs',0)
        
        #Initial design variables for a SCIG designed for a 5MW turbine
        self.SCIG.SCIG_r_s= 0.55 #meter
        self.SCIG.SCIG_l_s= 1.3 #meter
        self.SCIG.SCIG_h_s = 0.09 #meter
        self.SCIG.SCIG_h_r = 0.050 #meter
        self.SCIG.SCIG_I_0 = 140   #Ampere
        self.SCIG.SCIG_B_symax = 1.4  #Tesla
        self.SCIG.Eta_target=93
        self.SCIG.SCIG_P_rated=5e6
        self.SCIG.Gearbox_efficiency=0.955
        self.SCIG.SCIG_N_rated=1200
        
        # Specific costs
        self.SCIG.C_Cu   =4.786                  # Unit cost of Copper $/kg
        self.SCIG.C_Fe	= 0.556                    # Unit cost of Iron $/kg
        self.SCIG.C_Fes =0.50139                   # specific cost of structure
        
        #Material properties
        self.SCIG.rho_Fe = 7700                 #Steel density
        self.SCIG.rho_Copper =8900                  # Kg/m3 copper density
	
	
    def test_functionality(self):
        
        self.SCIG.run()
        
        self.assertEqual(round(self.SCIG.Mass/1000,1), 23.7)
        
        
class Test_DFIG(unittest.TestCase):

    def setUp(self):

        self.DFIG = DFIG_Opt('CONMINdriver','DFIG_Cost.Costs',0)
        
        #Initial design variables for a DFIG designed for a 5MW turbine
        
        self.DFIG.Eta_target=93
        self.DFIG.DFIG_P_rated=5e6
        self.DFIG.Gearbox_efficiency=0.955
        self.DFIG.DFIG_r_s= 0.61 #meter
        self.DFIG.DFIG_l_s= 0.49 #meter
        self.DFIG.DFIG_h_s = 0.08 #meter
        self.DFIG.DFIG_h_r = 0.1 #meter
        self.DFIG.DFIG_I_0 = 40 #Ampere
        self.DFIG.DFIG_B_symax = 1.3 #Tesla
        self.DFIG.DFIG_S_Nmax = -0.2  #Tesla
        self.DFIG.DFIG_N_rated=1200
        
        # Specific costs
        self.DFIG.C_Cu   =4.786                  # Unit cost of Copper $/kg
        self.DFIG.C_Fe	= 0.556                    # Unit cost of Iron $/kg
        self.DFIG.C_Fes =0.50139                   # specific cost of structure
        
        #Material properties
        self.DFIG.rho_Fe = 7700                 #Steel density
        self.DFIG.rho_Copper =8900                  # Kg/m3 copper density
	
	
    def test_functionality(self):
        
        self.DFIG.run()
        
        self.assertEqual(round(self.DFIG.Mass/1000,1), 25.2)



if __name__ == "__main__":
    unittest.main()
    


from wisdem.lcoe.lcoe_se_assembly import lcoe_se_assembly
import numpy as np
import unittest
# NREL 5 MW in land-based wind plant with high winds (as class I)
class test_lcoe_se(unittest.TestCase):
  def test1(self):
    wind_class = 'I'
    sea_depth = 0.0
    with_new_nacelle = True
    with_landbos = False
    flexible_blade = False
    with_3pt_drive = False
    with_ecn_opex = False
    ecn_file = ''
    with_openwind=False
    ow_file=None
    ow_wkbook=None
    
    lcoe_se = lcoe_se_assembly(with_new_nacelle,with_landbos,flexible_blade,with_3pt_drive,with_ecn_opex,ecn_file)
    
    # === Set assembly variables and objects ===
    lcoe_se.sea_depth = sea_depth # 0.0 for land-based turbine
    lcoe_se.turbine_number = 100
    lcoe_se.year = 2009
    lcoe_se.month = 12
    
    rotor = lcoe_se.rotor
    nacelle = lcoe_se.nacelle
    tower = lcoe_se.tower
    tcc_a = lcoe_se.tcc_a
    # bos_a = lcoe_se.bos_a
    # opex_a = lcoe_se.opex_a
    aep_a = lcoe_se.aep_a
    fin_a = lcoe_se.fin_a
    
    # Turbine ===========
    from wisdem.reference_turbines.nrel5mw.nrel5mw import configure_nrel5mw_turbine
    configure_nrel5mw_turbine(lcoe_se,wind_class,lcoe_se.sea_depth)
    
    # tcc ====
    lcoe_se.advanced_blade = True
    lcoe_se.offshore = False
    lcoe_se.assemblyCostMultiplier = 0.30
    lcoe_se.profitMultiplier = 0.20
    lcoe_se.overheadCostMultiplier = 0.0
    lcoe_se.transportMultiplier = 0.0
    
    # for new landBOS
    # === new landBOS ===
    if with_landbos:
        lcoe_se.voltage = 137
        lcoe_se.distInter = 5
        lcoe_se.terrain = 'FLAT_TO_ROLLING'
        lcoe_se.layout = 'SIMPLE'
        lcoe_se.soil = 'STANDARD'
    
    # aep ==== # based on COE review for land-based machines
    if not with_openwind:
        lcoe_se.array_losses = 0.059
        lcoe_se.A = 8.9 # weibull of 7.25 at 50 m with shear exp of 0.143
        lcoe_se.k = 2.0
    lcoe_se.other_losses = 0.101
    if not with_ecn_opex:
        lcoe_se.availability = 0.94
    
    # fin ===
    lcoe_se.fixed_charge_rate = 0.095
    lcoe_se.construction_finance_rate = 0.0
    lcoe_se.tax_rate = 0.4
    lcoe_se.discount_rate = 0.07
    lcoe_se.construction_time = 1.0
    lcoe_se.project_lifetime = 20.0
    
    # Set plant level inputs ===
    shearExp = 0.2 #TODO : should be an input to lcoe
    #rotor.cdf_reference_height_wind_speed = 90.0
    if not with_openwind:
        lcoe_se.array_losses = 0.1
    lcoe_se.other_losses = 0.0
    if not with_ecn_opex:
        lcoe_se.availability = 0.98
    rotor.turbulence_class = 'B'
    lcoe_se.multiplier = 2.23
    
    if wind_class == 'Offshore':
        # rotor.cdf_reference_mean_wind_speed = 8.4 # TODO - aep from its own module
        # rotor.cdf_reference_height_wind_speed = 50.0
        # rotor.weibull_shape = 2.1
        shearExp = 0.14 # TODO : should be an input to lcoe
        lcoe_se.array_losses = 0.15
        if not with_ecn_opex:
            lcoe_se.availability = 0.96
        lcoe_se.offshore = True
        lcoe_se.multiplier = 2.33
        lcoe_se.fixed_charge_rate = 0.118
    
    rotor.shearExp = shearExp
    tower.wind1.shearExp = shearExp
    tower.wind2.shearExp = shearExp
    
    # ====
    
    # === Run default assembly and print results
    lcoe_se.run()
    # ====
    self.assertEqual(np.round(lcoe_se.rotor.mass_all_blades, 2), 54674.80)
    self.assertEqual(np.round(lcoe_se.maxdeflection.ground_clearance, 2), 28.47)
    # === Print ===
    
    print "Key Turbine Outputs for NREL 5 MW Reference Turbine"
    print 'mass rotor blades:{0:.2f} (kg) '.format(lcoe_se.rotor.mass_all_blades)
    print 'mass hub system: {0:.2f} (kg) '.format(lcoe_se.hub.hub_system_mass)
    print 'mass nacelle: {0:.2f} (kg) '.format(lcoe_se.nacelle.nacelle_mass)
    print 'mass tower: {0:.2f} (kg) '.format(lcoe_se.tower.mass)
    print 'maximum tip deflection: {0:.2f} (m) '.format(lcoe_se.maxdeflection.max_tip_deflection)
    print 'ground clearance: {0:.2f} (m) '.format(lcoe_se.maxdeflection.ground_clearance)
    print
    print "Key Plant Outputs for wind plant with NREL 5 MW Turbine"
    #print "LCOE: ${0:.4f} USD/kWh".format(lcoe_se.lcoe) # not in base output set (add to assembly output if desired)
    print "COE: ${0:.4f} USD/kWh".format(lcoe_se.coe)
    print
    print "AEP per turbine: {0:.1f} kWh/turbine".format(lcoe_se.net_aep / lcoe_se.turbine_number)
    print "Turbine Cost: ${0:.2f} USD".format(lcoe_se.turbine_cost)
    print "BOS costs per turbine: ${0:.2f} USD/turbine".format(lcoe_se.bos_costs / lcoe_se.turbine_number)
    print "OPEX per turbine: ${0:.2f} USD/turbine".format(lcoe_se.avg_annual_opex / lcoe_se.turbine_number)    
    
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
WindWaveDrag.py

Created by RRD on 2015-07-13.
Copyright (c) NREL. All rights reserved.
"""

#-------------------------------------------------------------------------------
# Name:        WindWaveDrag.py
# Purpose:     It contains OpenMDAO's Components to calculate wind or wave drag
#              on cylinders.
#
# Author:      ANing/RRD
#
# Created:     13/07/2015 - It is based on load function calculations developed for tower and jacket.
#                             Reestablished elements needed by jacketSE that were removed. Changed names to vartrees.
# Copyright:   (c) rdamiani 2015
# Licence:     <Apache 2015>
#-------------------------------------------------------------------------------
from __future__ import print_function
import math
import numpy as np

from openmdao.api import ExplicitComponent, Problem, Group
from wisdem.commonse.utilities import sind, cosd  # , linspace_with_deriv, interp_with_deriv, hstack, vstack
from wisdem.commonse.csystem import DirectionVector

from wisdem.commonse.akima import Akima

#TODO CHECK

# -----------------
#  Helper Functions
# -----------------
# "Experiments on the Flow Past a Circular Cylinder at Very High Reynolds Numbers", Roshko
Re_pt = [0.00001, 0.0001, 0.0010, 0.0100, 0.0200, 0.1220, 0.2000, 0.3000, 0.4000,
         0.5000, 1.0000, 1.5000, 2.0000, 2.5000, 3.0000, 3.5000, 4.0000, 5.0000, 10.0000]
cd_pt = [4.0000,  2.0000, 1.1100, 1.1100, 1.2000, 1.2000, 1.1700, 0.9000, 0.5400,
         0.3100, 0.3800, 0.4600, 0.5300, 0.5700, 0.6100, 0.6400, 0.6700, 0.7000, 0.7000]

drag_spline = Akima(np.log10(Re_pt), cd_pt, delta_x=0.0)  # exact akima because control points do not change

def cylinderDrag(Re):
    """Drag coefficient for a smooth circular cylinder.

    Parameters
    ----------
    Re : array_like
        Reynolds number

    Returns
    -------
    cd : array_like
        drag coefficient (normalized by cylinder diameter)

    """

    ReN = Re / 1.0e6

    cd = np.zeros_like(Re)
    dcd_dRe = np.zeros_like(Re)
    idx = ReN > 0
    cd[idx], dcd_dRe[idx], _, _ = drag_spline.interp(np.log10(ReN[idx]))
    dcd_dRe[idx] /= (Re[idx]*math.log(10))  # chain rule

    return cd, dcd_dRe

# -----------------
#  Components
# -----------------

class AeroHydroLoads(ExplicitComponent):
    def initialize(self):
        self.options.declare('nPoints')
        
    def setup(self):
        nPoints = self.options['nPoints']

        #inputs

        self.add_input('windLoads_Px', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in x-direction')
        self.add_input('windLoads_Py', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in y-direction')
        self.add_input('windLoads_Pz', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in z-direction')
        self.add_input('windLoads_qdyn', np.zeros(nPoints), units='N/m**2', desc='dynamic pressure')
        self.add_input('windLoads_z', np.zeros(nPoints), units='m', desc='corresponding heights')
        self.add_input('windLoads_d', np.zeros(nPoints), units='m', desc='corresponding diameters')
        self.add_input('windLoads_beta', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')
        #self.add_input('windLoads_Px0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_input('windLoads_Py0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_input('windLoads_Pz0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_input('windLoads_qdyn0', 0.0, units='N/m**2', desc='dynamic pressure at z=0 MSL')
        #self.add_input('windLoads_beta0', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')


        self.add_input('waveLoads_Px', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in x-direction')
        self.add_input('waveLoads_Py', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in y-direction')
        self.add_input('waveLoads_Pz', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in z-direction')
        self.add_input('waveLoads_qdyn', np.zeros(nPoints), units='N/m**2', desc='dynamic pressure')
        self.add_input('waveLoads_z', np.zeros(nPoints), units='m', desc='corresponding heights')
        self.add_input('waveLoads_d', np.zeros(nPoints), units='m', desc='corresponding diameters')
        self.add_input('waveLoads_beta', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')
        #self.add_input('waveLoads_Px0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_input('waveLoads_Py0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_input('waveLoads_Pz0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_input('waveLoads_qdyn0', 0.0, units='N/m**2', desc='dynamic pressure at z=0 MSL')
        #self.add_input('waveLoads_beta0', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')

        self.add_input('z', np.zeros(nPoints), units='m', desc='locations along cylinder')
        self.add_input('yaw', 0.0, units='deg', desc='yaw angle')

        #outputs
        self.add_output('Px', np.zeros(nPoints), units='N/m', desc='force per unit length in x-direction')
        self.add_output('Py', np.zeros(nPoints), units='N/m', desc='force per unit length in y-direction')
        self.add_output('Pz', np.zeros(nPoints), units='N/m', desc='force per unit length in z-direction')
        self.add_output('qdyn', np.zeros(nPoints), units='N/m**2', desc='dynamic pressure')

    def compute(self, inputs, outputs):
        # aero/hydro loads
        # wind = inputs['windLoads']
        # wave = inputs['waveLoads']
        # outloads = inputs['outloads']
        z = inputs['z']
        hubHt = z[-1]  # top of cylinder
        windLoads = DirectionVector(inputs['windLoads_Px'], inputs['windLoads_Py'], inputs['windLoads_Pz']).inertialToWind(inputs['windLoads_beta']).windToYaw(inputs['yaw'])
        waveLoads = DirectionVector(inputs['waveLoads_Px'], inputs['waveLoads_Py'], inputs['waveLoads_Pz']).inertialToWind(inputs['waveLoads_beta']).windToYaw(inputs['yaw'])

        Px = np.interp(z, inputs['windLoads_z'], windLoads.x) + np.interp(z, inputs['waveLoads_z'], waveLoads.x)
        Py = np.interp(z, inputs['windLoads_z'], windLoads.y) + np.interp(z, inputs['waveLoads_z'], waveLoads.y)
        Pz = np.interp(z, inputs['windLoads_z'], windLoads.z) + np.interp(z, inputs['waveLoads_z'], waveLoads.z)
        qdyn = np.interp(z, inputs['windLoads_z'], inputs['windLoads_qdyn']) + np.interp(z, inputs['waveLoads_z'], inputs['waveLoads_qdyn'])
        # outloads.z = z

        #The following are redundant, at one point we will consolidate them to something that works for both cylinder (not using vartrees) and jacket (still using vartrees)
        outputs['Px'] = Px
        outputs['Py'] = Py
        outputs['Pz'] = Pz
        outputs['qdyn'] = qdyn

# -----------------

class CylinderWindDrag(ExplicitComponent):
    """drag forces on a cylindrical cylinder due to wind"""

    def initialize(self):
        self.options.declare('nPoints')
        
    def setup(self):
        nPoints = self.options['nPoints']

        # variables
        self.add_input('U', np.zeros(nPoints), units='m/s', desc='magnitude of wind speed')
        self.add_input('z', np.zeros(nPoints), units='m', desc='heights where wind speed was computed')
        self.add_input('d', np.zeros(nPoints), units='m', desc='corresponding diameter of cylinder section')

        # parameters
        self.add_input('beta', 0.0, units='deg', desc='corresponding wind angles relative to inertial coordinate system')
        self.add_input('rho', 0.0, units='kg/m**3', desc='air density')
        self.add_input('mu', 0.0, units='kg/(m*s)', desc='dynamic viscosity of air')
        #TODO not sure what to do here?
        self.add_input('cd_usr', -1., desc='User input drag coefficient to override Reynolds number based one')

        # out
        self.add_output('windLoads_Px', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in x-direction')
        self.add_output('windLoads_Py', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in y-direction')
        self.add_output('windLoads_Pz', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in z-direction')
        self.add_output('windLoads_qdyn', np.zeros(nPoints), units='N/m**2', desc='dynamic pressure')
        self.add_output('windLoads_z', np.zeros(nPoints), units='m', desc='corresponding heights')
        self.add_output('windLoads_d', np.zeros(nPoints), units='m', desc='corresponding diameters')
        self.add_output('windLoads_beta', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')
        #self.add_output('windLoads_Px0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_output('windLoads_Py0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_output('windLoads_Pz0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_output('windLoads_qdyn0', 0.0, units='N/m**2', desc='dynamic pressure at z=0 MSL')
        #self.add_output('windLoads_beta0', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')

        self.declare_partials('*', '*')

    def compute(self, inputs, outputs):

        rho = inputs['rho']
        U = inputs['U']
        d = inputs['d']
        mu = inputs['mu']
        beta = inputs['beta']

        # dynamic pressure
        q = 0.5*rho*U**2

        # Reynolds number and drag
        if float(inputs['cd_usr']) < 0.:
            Re = rho*U*d/mu
            cd, dcd_dRe = cylinderDrag(Re)
        else:
            cd = inputs['cd_usr']
            Re = 1.0
            dcd_dRe = 0.0
        Fp = q*cd*d

        # components of distributed loads
        Px = Fp*cosd(beta)
        Py = Fp*sind(beta)
        Pz = 0*Fp

        # pack data
        outputs['windLoads_Px'] = Px
        outputs['windLoads_Py'] = Py
        outputs['windLoads_Pz'] = Pz
        outputs['windLoads_qdyn'] = q
        outputs['windLoads_z'] = inputs['z']
        outputs['windLoads_beta'] = beta


    def compute_partials(self, inputs, J):

        # rename
        rho = inputs['rho']
        U = inputs['U']
        d = inputs['d']
        mu = inputs['mu']
        beta = inputs['beta']

        # dynamic pressure
        q = 0.5*rho*U**2

        # Reynolds number and drag
        if float(inputs['cd_usr']) < 0.:
            Re = rho*U*d/mu
            cd, dcd_dRe = cylinderDrag(Re)
        else:
            cd = inputs['cd_usr']
            Re = 1.0
            dcd_dRe = 0.0

        # derivatives
        dq_dU = rho*U
        const = (dq_dU*cd + q*dcd_dRe*rho*d/mu)*d
        dPx_dU = const*cosd(beta)
        dPy_dU = const*sind(beta)

        const = (cd + dcd_dRe*Re)*q
        dPx_dd = const*cosd(beta)
        dPy_dd = const*sind(beta)

        n = len(inputs['z'])

        zeron = np.zeros((n, n))
        
        J['windLoads_Px', 'U'] = np.diag(dPx_dU)
        J['windLoads_Px', 'z'] = zeron
        J['windLoads_Px', 'd'] = np.diag(dPx_dd)

        J['windLoads_Py', 'U'] = np.diag(dPy_dU)
        J['windLoads_Py', 'z'] = zeron
        J['windLoads_Py', 'd'] = np.diag(dPy_dd)

        J['windLoads_Pz', 'U'] = zeron
        J['windLoads_Pz', 'z'] = zeron
        J['windLoads_Pz', 'd'] = zeron

        J['windLoads_qdyn', 'U'] = np.diag(dq_dU)
        J['windLoads_qdyn', 'z'] = zeron
        J['windLoads_qdyn', 'd'] = zeron

        J['windLoads_z', 'U'] = zeron
        J['windLoads_z', 'z'] = np.eye(n)
        J['windLoads_z', 'd'] = zeron

        

# -----------------

class CylinderWaveDrag(ExplicitComponent):
    """drag forces on a cylindrical cylinder due to waves"""


    def initialize(self):
        self.options.declare('nPoints')
        
    def setup(self):
        nPoints = self.options['nPoints']

        # variables
        self.add_input('U', np.zeros(nPoints), units='m/s', desc='magnitude of wave speed')
        #self.add_input('U0', 0.0, units='m/s', desc='magnitude of wave speed at z=0 MSL')
        self.add_input('A', np.zeros(nPoints), units='m/s**2', desc='magnitude of wave acceleration')
        self.add_input('p', np.zeros(nPoints), units='N/m**2', desc='pressure oscillation')
        #self.add_input('A0', 0.0, units='m/s**2', desc='magnitude of wave acceleration at z=0 MSL')
        self.add_input('z', np.zeros(nPoints), units='m', desc='heights where wave speed was computed')
        self.add_input('d', np.zeros(nPoints), units='m', desc='corresponding diameter of cylinder section')

        # parameters
        #self.add_input('wlevel', 0.0, units='m', desc='Water Level, to assess z w.r.t. MSL')
        self.add_input('beta', 0.0, units='deg', desc='corresponding wave angles relative to inertial coordinate system')
        #self.add_input('beta0', 0.0, units='deg', desc='corresponding wave angles relative to inertial coordinate system at z=0 MSL')
        self.add_input('rho', 0.0, units='kg/m**3', desc='water density')
        self.add_input('mu', 0.0, units='kg/(m*s)', desc='dynamic viscosity of water')
        self.add_input('cm', 0.0, desc='mass coefficient')
        #TODO not sure what to do here?
        self.add_input('cd_usr', -1., desc='User input drag coefficient to override Reynolds number based one')

        # out
        self.add_output('waveLoads_Px', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in x-direction')
        self.add_output('waveLoads_Py', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in y-direction')
        self.add_output('waveLoads_Pz', np.zeros(nPoints), units='N/m', desc='distributed loads, force per unit length in z-direction')
        self.add_output('waveLoads_qdyn', np.zeros(nPoints), units='N/m**2', desc='dynamic pressure')
        self.add_output('waveLoads_pt', np.zeros(nPoints), units='N/m**2', desc='total (static+dynamic) pressure')
        self.add_output('waveLoads_z', np.zeros(nPoints), units='m', desc='corresponding heights')
        self.add_output('waveLoads_d', np.zeros(nPoints), units='m', desc='corresponding diameters')
        self.add_output('waveLoads_beta', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')
        #self.add_output('waveLoads_Px0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_output('waveLoads_Py0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_output('waveLoads_Pz0', 0.0, units='N/m', desc='Distributed load at z=0 MSL')
        #self.add_output('waveLoads_qdyn0', 0.0, units='N/m**2', desc='dynamic pressure at z=0 MSL')
        #self.add_output('waveLoads_beta0', 0.0, units='deg', desc='wind/wave angle relative to inertia c.s.')

        self.declare_partials('*', '*')

    def compute(self, inputs, outputs):

        #wlevel = inputs['wlevel']
        #if wlevel > 0.0: wlevel *= -1.0
        
        rho = inputs['rho']
        U = inputs['U']
        #U0 = inputs['U0']
        d = inputs['d']
        #zrel= inputs['z']-wlevel
        mu = inputs['mu']
        beta = inputs['beta']
        #beta0 = inputs['beta0']

        # dynamic pressure
        q = 0.5*rho*U**2
        #q0= 0.5*rho*U0**2

        # Reynolds number and drag
        if float(inputs['cd_usr']) < 0.:
            Re = rho*U*d/mu
            cd, dcd_dRe = cylinderDrag(Re)
        else:
            cd = inputs['cd_usr']*np.ones_like(d)
            Re = 1.0
            dcd_dRe = 0.0

        # inertial and drag forces
        Fi = rho*inputs['cm']*math.pi/4.0*d**2*inputs['A']  # Morrison's equation
        Fd = q*cd*d
        Fp = Fi + Fd

        # components of distributed loads
        Px = Fp*cosd(beta)
        Py = Fp*sind(beta)
        Pz = 0.*Fp

        #FORCES [N/m] AT z=0 m
        #idx0 = np.abs(zrel).argmin()  # closest index to z=0, used to find d at z=0
        #d0 = d[idx0]  # initialize
        #cd0 = cd[idx0]  # initialize
        #if (zrel[idx0]<0.) and (idx0< (zrel.size-1)):       # point below water
        #    d0 = np.mean(d[idx0:idx0+2])
        #    cd0 = np.mean(cd[idx0:idx0+2])
        #elif (zrel[idx0]>0.) and (idx0>0):     # point above water
        #    d0 = np.mean(d[idx0-1:idx0+1])
        #    cd0 = np.mean(cd[idx0-1:idx0+1])
        #Fi0 = rho*inputs['cm']*math.pi/4.0*d0**2*inputs['A0']  # Morrison's equation
        #Fd0 = q0*cd0*d0
        #Fp0 = Fi0 + Fd0

        #Px0 = Fp0*cosd(beta0)
        #Py0 = Fp0*sind(beta0)
        #Pz0 = 0.*Fp0

        #Store qties at z=0 MSL
        #outputs['waveLoads_Px0'] = Px0
        #outputs['waveLoads_Py0'] = Py0
        #outputs['waveLoads_Pz0'] = Pz0
        #outputs['waveLoads_qdyn0'] = q0
        #outputs['waveLoads_beta0'] = beta0

        # pack data
        outputs['waveLoads_Px'] = Px
        outputs['waveLoads_Py'] = Py
        outputs['waveLoads_Pz'] = Pz
        outputs['waveLoads_qdyn'] = q
        outputs['waveLoads_pt'] = q + inputs['p']
        outputs['waveLoads_z'] = inputs['z']
        outputs['waveLoads_beta'] = beta
        outputs['waveLoads_d'] = d


    def compute_partials(self, inputs, J):

        #wlevel = inputs['wlevel']
        #if wlevel > 0.0: wlevel *= -1.0
        
        rho = inputs['rho']
        U = inputs['U']
        #U0 = inputs['U0']
        d = inputs['d']
        #zrel= inputs['z']-wlevel
        mu = inputs['mu']
        beta = inputs['beta']
        #beta0 = inputs['beta0']

        # dynamic pressure
        q = 0.5*rho*U**2
        #q0= 0.5*rho*U0**2

        # Reynolds number and drag
        if float(inputs['cd_usr']) < 0.:
            cd = inputs['cd_usr']*np.ones_like(d)
            Re = 1.0
            dcd_dRe = 0.0
        else:
            Re = rho*U*d/mu
            cd, dcd_dRe = cylinderDrag(Re)

        # derivatives
        dq_dU = rho*U
        const = (dq_dU*cd + q*dcd_dRe*rho*d/mu)*d
        dPx_dU = const*cosd(beta)
        dPy_dU = const*sind(beta)

        const = (cd + dcd_dRe*Re)*q + rho*inputs['cm']*math.pi/4.0*2*d*inputs['A']
        dPx_dd = const*cosd(beta)
        dPy_dd = const*sind(beta)

        const = rho*inputs['cm']*math.pi/4.0*d**2
        dPx_dA = const*cosd(beta)
        dPy_dA = const*sind(beta)

        n = len(inputs['z'])

        zeron = np.zeros((n, n))
        
        J['waveLoads.Px', 'U'] = np.diag(dPx_dU)
        J['waveLoads.Px', 'A'] = np.diag(dPx_dA)
        J['waveLoads.Px', 'z'] = zeron
        J['waveLoads.Px', 'd'] = np.diag(dPx_dd)
        J['waveLoads.Px', 'p'] = zeron

        J['waveLoads.Py', 'U'] = np.diag(dPy_dU)
        J['waveLoads.Py', 'A'] = np.diag(dPy_dA)
        J['waveLoads.Py', 'z'] = zeron
        J['waveLoads.Py', 'd'] = np.diag(dPy_dd)
        J['waveLoads.Py', 'p'] = zeron

        J['waveLoads.Pz', 'U'] = zeron
        J['waveLoads.Pz', 'A'] = zeron
        J['waveLoads.Pz', 'z'] = zeron
        J['waveLoads.Pz', 'd'] = zeron
        J['waveLoads.Pz', 'p'] = zeron

        J['waveLoads.qdyn', 'U'] = np.diag(dq_dU)
        J['waveLoads.qdyn', 'A'] = zeron
        J['waveLoads.qdyn', 'z'] = zeron
        J['waveLoads.qdyn', 'd'] = zeron
        J['waveLoads.qdyn', 'p'] = zeron

        J['waveLoads.pt', 'U'] = np.diag(dq_dU)
        J['waveLoads.pt', 'A'] = zeron
        J['waveLoads.pt', 'z'] = zeron
        J['waveLoads.pt', 'd'] = zeron
        J['waveLoads.pt', 'p'] = 1.0

        J['waveLoads.z', 'U'] = zeron
        J['waveLoads.z', 'A'] = zeron
        J['waveLoads.z', 'z'] = np.eye(n)
        J['waveLoads.z', 'd'] = zeron
        J['waveLoads.z', 'p'] = zeron

        

#___________________________________________#

def main():
    # initialize problem
    U = np.array([20., 25., 30.])
    z = np.array([10., 30., 80.])
    d = np.array([5.5, 4., 3.])

    beta = np.array([45., 45., 45.])
    rho = 1.225
    mu = 1.7934e-5
    #cd_usr = 0.7

    nPoints = len(z)


    prob = Problem()

    root = prob.model = Group()

    root.add('p1', CylinderWindDrag(nPoints))

    prob.setup()

    prob['p1.U'] = U
    prob['p1.z'] = z
    prob['p1.d'] = d
    prob['p1.beta'] = beta
    prob['p1.rho'] = rho
    prob['p1.mu'] = mu
    #prob['p1.cd_usr'] = cd_usr

    #run
    prob.run_once()

    # out
    Re = prob['p1.rho']*prob['p1.U']*prob['p1.d']/prob['p1.mu']
    cd, dcd_dRe = cylinderDrag(Re)
    print(cd)
    import matplotlib.pyplot as plt

    plt.plot(prob['p1.windLoads_Px'], prob['p1.windLoads_z'])
    plt.plot(prob['p1.windLoads_Py'], prob['p1.windLoads_z'])
    plt.plot(prob['p1.windLoads_qdyn'], prob['p1.windLoads_z'])
    plt.show()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8
"""
test_tower_gradients.py

Created by Andrew Ning on 2013-12-20.
Copyright (c) NREL. All rights reserved.
"""

import unittest
import numpy as np
from wisdem.commonse.rna import RNAMass, RotorLoads
# --- tower setup ------
from wisdem.commonse.environment import PowerWind, LogWind
from wisdem.towerse.tower import TowerSE, TowerWindDrag, TowerWaveDrag, TowerDiscretization, GeometricConstraints#, JacketPositioning
from openmdao.api import Problem, Group, IndepVarComp, pyOptSparseDriver


class TotalDerivTestsFlorisAEPOpt(unittest.TestCase):

    def setUp(self):

        # --- geometry ----
        # --- geometry ----
        z_param = np.array([0.0, 43.8, 87.6])
        d_param = np.array([6.0, 4.935, 3.87])
        t_param = np.array([0.027*1.3, 0.023*1.3, 0.019*1.3])
        n = 15
        z_full = np.linspace(0.0, 87.6, n)
        L_reinforced = 30.0*np.ones(n)  # [m] buckling length
        theta_stress = 0.0*np.ones(n)
        yaw = 0.0

        # --- material props ---
        E = 210e9*np.ones(n)
        G = 80.8e9*np.ones(n)
        rho = 8500.0*np.ones(n)
        sigma_y = 450.0e6*np.ones(n)

        # --- spring reaction data.  Use float('inf') for rigid constraints. ---
        kidx = np.array([0], dtype=int)  # applied at base
        kx = np.array([float('inf')])
        ky = np.array([float('inf')])
        kz = np.array([float('inf')])
        ktx = np.array([float('inf')])
        kty = np.array([float('inf')])
        ktz = np.array([float('inf')])
        nK = len(kidx)

        # --- extra mass ----
        midx = np.array([n-1], dtype=int)  # RNA mass at top
        m = np.array([285598.8])
        mIxx = np.array([1.14930678e+08])
        mIyy = np.array([2.20354030e+07])
        mIzz = np.array([1.87597425e+07])
        mIxy = np.array([0.00000000e+00])
        mIxz = np.array([5.03710467e+05])
        mIyz = np.array([0.00000000e+00])
        mrhox = np.array([-1.13197635])
        mrhoy = np.array([0.])
        mrhoz = np.array([0.50875268])
        nMass = len(midx)
        addGravityLoadForExtraMass = True
        # -----------

        # --- wind ---
        wind_zref = 90.0
        wind_z0 = 0.0
        shearExp = 0.2
        # ---------------

        # if addGravityLoadForExtraMass=True be sure not to double count by adding those force here also
        # # --- loading case 1: max Thrust ---
        wind_Uref1 = 11.73732
        plidx1 = np.array([n-1], dtype=int)  # at  top
        Fx1 = np.array([1284744.19620519])
        Fy1 = np.array([0.])
        Fz1 = np.array([-2914124.84400512])
        Mxx1 = np.array([3963732.76208099])
        Myy1 = np.array([-2275104.79420872])
        Mzz1 = np.array([-346781.68192839])
        nPL = len(plidx1)
        # # ---------------

        # # --- loading case 2: max wind speed ---
        wind_Uref2 = 70.0
        plidx2 = np.array([n-1], dtype=int)  # at  top
        Fx2 = np.array([930198.60063279])
        Fy2 = np.array([0.])
        Fz2 = np.array([-2883106.12368949])
        Mxx2 = np.array([-1683669.22411597])
        Myy2 = np.array([-2522475.34625363])
        Mzz2 = np.array([147301.97023764])
        # # ---------------

        # --- safety factors ---
        gamma_f = 1.35
        gamma_m = 1.3
        gamma_n = 1.0
        gamma_b = 1.1
        # ---------------

        # --- fatigue ---
        z_DEL = np.array([0.000, 1.327, 3.982, 6.636, 9.291, 11.945, 14.600, 17.255, 19.909, 22.564, 25.218, 27.873, 30.527, 33.182, 35.836, 38.491, 41.145, 43.800, 46.455, 49.109, 51.764, 54.418, 57.073, 59.727, 62.382, 65.036, 67.691, 70.345, 73.000, 75.655, 78.309, 80.964, 83.618, 86.273, 87.600])
        M_DEL = 1e3*np.array([8.2940E+003, 8.1518E+003, 7.8831E+003, 7.6099E+003, 7.3359E+003, 7.0577E+003, 6.7821E+003, 6.5119E+003, 6.2391E+003, 5.9707E+003, 5.7070E+003, 5.4500E+003, 5.2015E+003, 4.9588E+003, 4.7202E+003, 4.4884E+003, 4.2577E+003, 4.0246E+003, 3.7942E+003, 3.5664E+003, 3.3406E+003, 3.1184E+003, 2.8977E+003, 2.6811E+003, 2.4719E+003, 2.2663E+003, 2.0673E+003, 1.8769E+003, 1.7017E+003, 1.5479E+003, 1.4207E+003, 1.3304E+003, 1.2780E+003, 1.2673E+003, 1.2761E+003])
        nDEL = len(z_DEL)
        gamma_fatigue = 1.35*1.3*1.0
        life = 20.0
        m_SN = 4
        # ---------------


        # --- constraints ---
        min_d_to_t = 120.0
        min_taper = 0.4
        # ---------------

        # # V_max = 80.0  # tip speed
        # # D = 126.0
        # # .freq1p = V_max / (D/2) / (2*pi)  # convert to Hz

        nPoints = len(z_param)
        nFull = len(z_full)
        wind = 'PowerWind'

        prob = Problem()
        root = prob.root = Group()

        prob.driver = pyOptSparseDriver()
        prob.driver.options['optimizer'] = 'SNOPT'
        prob.driver.opt_settings['Major iterations limit'] = 1000

        root.add('z_param', IndepVarComp('z_param', z_param))
        root.add('d_param', IndepVarComp('d_param', d_param))
        root.add('t_param', IndepVarComp('t_param', t_param))
        root.add('TowerSE', TowerSE(nPoints, nFull, nK, nMass, nPL, nDEL, wind=wind))


        prob.driver.add_objective('TowerSE.tower1.mass', scaler=1E-6)
        prob.driver.add_desvar('z_param.z_param', lower=np.zeros(nPoints), upper=np.ones(nPoints)*1000., scaler=1E-2)
        prob.driver.add_desvar('t_param.t_param', lower=np.ones(nPoints)*0.001, upper=np.ones(nPoints)*1000., scaler=1E-6)
        prob.driver.add_desvar('d_param.d_param', np.array([2,2.1,2.2]), upper=np.ones(nPoints)*1000., scaler=1E-6)

        prob.root.connect('z_param.z_param', 'TowerSE.z_param')
        prob.root.connect('d_param.d_param', 'TowerSE.d_param')
        prob.root.connect('t_param.t_param', 'TowerSE.t_param')

        prob.driver.add_constraint('TowerSE.tower1.stress', upper=np.ones(n))
        prob.driver.add_constraint('TowerSE.tower2.stress', upper=np.ones(n))
        prob.driver.add_constraint('TowerSE.tower1.global_buckling', upper=np.ones(n))
        prob.driver.add_constraint('TowerSE.tower2.global_buckling', upper=np.ones(n))
        prob.driver.add_constraint('TowerSE.tower1.shell_buckling', upper=np.ones(n))
        prob.driver.add_constraint('TowerSE.tower2.shell_buckling', upper=np.ones(n))
        prob.driver.add_constraint('TowerSE.tower1.damage', upper=np.ones(n)*0.8)
        prob.driver.add_constraint('TowerSE.gc.weldability', upper=np.zeros(n))
        prob.driver.add_constraint('TowerSE.gc.manufacturability', upper=np.zeros(n))
        freq1p = 0.2  # 1P freq in Hz
        prob.driver.add_constraint('TowerSE.tower1.f1', lower=1.1*freq1p)
        prob.driver.add_constraint('TowerSE.tower2.f1', lower=1.1*freq1p)

        prob.setup()

        if wind=='PowerWind':
            prob['TowerSE.wind1.shearExp'] = shearExp
            prob['TowerSE.wind2.shearExp'] = shearExp

        # assign values to params

        # --- geometry ----
        #prob['TowerSE.z_param'] = z_param
        #prob['TowerSE.d_param'] = d_param
        #prob['TowerSE.t_param'] = t_param
        prob['TowerSE.z_full'] = z_full
        prob['TowerSE.tower1.L_reinforced'] = L_reinforced
        prob['TowerSE.distLoads1.yaw'] = yaw

        # --- material props ---
        prob['TowerSE.tower1.E'] = E
        prob['TowerSE.tower1.G'] = G
        prob['TowerSE.tower1.rho'] = rho
        prob['TowerSE.tower1.sigma_y'] = sigma_y

        # --- spring reaction data.  Use float('inf') for rigid constraints. ---
        prob['TowerSE.tower1.kidx'] = kidx
        prob['TowerSE.tower1.kx'] = kx
        prob['TowerSE.tower1.ky'] = ky
        prob['TowerSE.tower1.kz'] = kz
        prob['TowerSE.tower1.ktx'] = ktx
        prob['TowerSE.tower1.kty'] = kty
        prob['TowerSE.tower1.ktz'] = ktz

        # --- extra mass ----
        prob['TowerSE.tower1.midx'] = midx
        prob['TowerSE.tower1.m'] = m
        prob['TowerSE.tower1.mIxx'] = mIxx
        prob['TowerSE.tower1.mIyy'] = mIyy
        prob['TowerSE.tower1.mIzz'] = mIzz
        prob['TowerSE.tower1.mIxy'] = mIxy
        prob['TowerSE.tower1.mIxz'] = mIxz
        prob['TowerSE.tower1.mIyz'] = mIyz
        prob['TowerSE.tower1.mrhox'] = mrhox
        prob['TowerSE.tower1.mrhoy'] = mrhoy
        prob['TowerSE.tower1.mrhoz'] = mrhoz
        prob['TowerSE.tower1.addGravityLoadForExtraMass'] = addGravityLoadForExtraMass
        # -----------

        # --- wind ---
        prob['TowerSE.wind1.zref'] = wind_zref
        prob['TowerSE.wind1.z0'] = wind_z0
        # ---------------

        # # --- loading case 1: max Thrust ---
        prob['TowerSE.wind1.Uref'] = wind_Uref1
        prob['TowerSE.tower1.plidx'] = plidx1
        prob['TowerSE.tower1.Fx'] = Fx1
        prob['TowerSE.tower1.Fy'] = Fy1
        prob['TowerSE.tower1.Fz'] = Fz1
        prob['TowerSE.tower1.Mxx'] = Mxx1
        prob['TowerSE.tower1.Myy'] = Myy1
        prob['TowerSE.tower1.Mzz'] = Mzz1
        # # ---------------

        # # --- loading case 2: max Wind Speed ---
        prob['TowerSE.wind2.Uref'] = wind_Uref2
        prob['TowerSE.tower2.plidx'] = plidx2
        prob['TowerSE.tower2.Fx'] = Fx2
        prob['TowerSE.tower2.Fy'] = Fy2
        prob['TowerSE.tower2.Fz'] = Fz2
        prob['TowerSE.tower2.Mxx'] = Mxx2
        prob['TowerSE.tower2.Myy'] = Myy2
        prob['TowerSE.tower2.Mzz'] = Mzz2
        # # ---------------

        # --- safety factors ---
        prob['TowerSE.tower1.gamma_f'] = gamma_f
        prob['TowerSE.tower1.gamma_m'] = gamma_m
        prob['TowerSE.tower1.gamma_n'] = gamma_n
        prob['TowerSE.tower1.gamma_b'] = gamma_b
        # ---------------

        # --- fatigue ---
        prob['TowerSE.tower1.z_DEL'] = z_DEL
        prob['TowerSE.tower1.M_DEL'] = M_DEL
        prob['TowerSE.tower1.gamma_fatigue'] = gamma_fatigue
        prob['TowerSE.tower1.life'] = life
        prob['TowerSE.tower1.m_SN'] = m_SN
        # ---------------

        # --- constraints ---
        prob['TowerSE.gc.min_d_to_t'] = min_d_to_t
        prob['TowerSE.gc.min_taper'] = min_taper
        # ---------------

        # # --- run ---
        prob.run()

        print prob['TowerSE.gc.weldability']
        print prob['TowerSE.gc.manufacturability']

        self.J = prob.check_total_derivatives(out_stream=None)
        """
        self.connect('tower1.mass', 'mass')
        self.connect('tower1.f1', 'f1')
        self.connect('tower1.f2', 'f2')
        self.connect('tower1.top_deflection', 'top_deflection1')
        self.connect('tower2.top_deflection', 'top_deflection2')
        self.connect('tower1.stress', 'stress1')
        self.connect('tower2.stress', 'stress2')
        self.connect('tower1.global_buckling', 'global_buckling1')
        self.connect('tower2.global_buckling', 'global_buckling2')
        self.connect('tower1.shell_buckling', 'shell_buckling1')
        self.connect('tower2.shell_buckling', 'shell_buckling2')
        self.connect('tower1.damage', 'damage')
        self.connect('gc.weldability', 'weldability')
        self.connect('gc.manufacturability', 'manufacturability')
        """
    def testMass(self):
        np.testing.assert_allclose(self.J[('TowerSE.tower1.mass', 'z_param.z_param')]['J_fwd'], self.J[('TowerSE.tower1.mass', 'z_param.z_param')]['J_fd'], 1e-6, 1e-6)
        np.testing.assert_allclose(self.J[('TowerSE.tower1.mass', 'd_param.d_param')]['J_fwd'], self.J[('TowerSE.tower1.mass', 'd_param.d_param')]['J_fd'], 1e-6, 1e-6)
        np.testing.assert_allclose(self.J[('TowerSE.tower1.mass', 't_param.t_param')]['J_fwd'], self.J[('TowerSE.tower1.mass', 't_param.t_param')]['J_fd'], 1e-6, 1e-6)


"""
class TestTowerWindDrag(unittest.TestCase):

    def test1(self):

        twd = TowerWindDrag()
        twd.U = [0., 8.80496275, 10.11424623, 10.96861453, 11.61821801, 12.14846828, 12.59962946, 12.99412772, 13.34582791, 13.66394248, 13.95492553, 14.22348635, 14.47317364, 14.70673252, 14.92633314, 15.13372281, 15.33033057, 15.51734112, 15.69574825, 15.86639432, 16.03]
        twd.z = [0., 4.38, 8.76, 13.14, 17.52, 21.9, 26.28, 30.66, 35.04, 39.42, 43.8, 48.18, 52.56, 56.94, 61.32, 65.7, 70.08, 74.46, 78.84, 83.22, 87.6]
        twd.d = [6., 5.8935, 5.787, 5.6805, 5.574, 5.4675, 5.361, 5.2545, 5.148, 5.0415, 4.935, 4.8285, 4.722, 4.6155, 4.509, 4.4025, 4.296, 4.1895, 4.083, 3.9765, 3.87]
        twd.beta = [0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.]
        twd.rho = 1.225
        twd.mu = 1.7934e-05

        names, errors = check_gradient(twd)

        for name, err in zip(names, errors):

            if name == 'd_windLoads.Px[0] / d_U[0]':
                tol = 2e-5  # central difference not accurate right at Re=0
            else:
                tol = 1e-6

            try:
                self.assertLessEqual(err, tol)
            except AssertionError, e:
                print '*** error in:', name
                raise e



class TestTowerWaveDrag(unittest.TestCase):

    def test1(self):

        twd = TowerWaveDrag()
        twd.U = [0., 8.80496275, 10.11424623, 10.96861453, 11.61821801, 12.14846828, 12.59962946, 12.99412772, 13.34582791, 13.66394248, 13.95492553, 14.22348635, 14.47317364, 14.70673252, 14.92633314, 15.13372281, 15.33033057, 15.51734112, 15.69574825, 15.86639432, 16.03]
        twd.z = [0., 4.38, 8.76, 13.14, 17.52, 21.9, 26.28, 30.66, 35.04, 39.42, 43.8, 48.18, 52.56, 56.94, 61.32, 65.7, 70.08, 74.46, 78.84, 83.22, 87.6]
        twd.d = [6., 5.8935, 5.787, 5.6805, 5.574, 5.4675, 5.361, 5.2545, 5.148, 5.0415, 4.935, 4.8285, 4.722, 4.6155, 4.509, 4.4025, 4.296, 4.1895, 4.083, 3.9765, 3.87]
        twd.beta = [0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.]
        twd.rho = 1.225
        twd.mu = 1.7934e-05

        twd.A = 1.1*twd.U
        twd.cm = 2.0

        names, errors = check_gradient(twd)

        for name, err in zip(names, errors):

            if name == 'd_waveLoads.Px[0] / d_U[0]':
                tol = 2e-5  # central difference not accurate right at Re=0
            else:
                tol = 1e-6

            try:
                self.assertLessEqual(err, tol)
            except AssertionError, e:
                print '*** error in:', name
                raise e



class TestTowerDiscretization(unittest.TestCase):

    def test1(self):

        td = TowerDiscretization()
        td.towerHeight = 87.6
        td.z = np.array([0.0, 0.5, 1.0])
        td.d = np.array([6.0, 4.935, 3.87])
        td.t = np.array([0.0351, 0.0299, 0.0247])
        td.L_reinforced = np.array([29.2, 29.2, 29.2])
        td.n = np.array([10, 7])
        # td.n_reinforced = 3

        check_gradient_unit_test(self, td)


    def test2(self):

        td = TowerDiscretization()
        td.towerHeight = np.random.rand(1)[0]
        td.z = np.array([0.0, 0.5, 1.0])
        td.d = np.random.rand(3)
        td.t = np.random.rand(3)
        td.L_reinforced = np.array([29.2, 29.2, 29.2])
        td.n = np.array([10, 7])
        # td.n_reinforced = 3

        check_gradient_unit_test(self, td)


    def test3(self):

        td = TowerDiscretization()
        td.towerHeight = 87.6
        td.z = np.array([0.0, 0.5, 1.0])
        td.d = np.array([6.0, 4.935, 3.87])
        td.t = np.array([0.0351, 0.0299, 0.0247])
        td.L_reinforced = np.array([29.2, 29.2, 29.2])
        td.n = np.array([10, 7])

        td.monopileHeight = 30.0
        td.d_monopile = 6.0
        td.t_monopile = 0.0351
        td.n_monopile = 5

        check_gradient_unit_test(self, td)

class TestJacketPositioning(unittest.TestCase):

    def test1(self):

        jp = JacketPositioning()
        # inputs
        jp.sea_depth = 20.0
        jp.tower_length = 87.6
        jp.tower_to_shaft = 2.0
        jp.monopile_extension = 5.0
        jp.deck_height = 15.0
        jp.d_monopile = 6.0
        jp.t_monopile = 0.06
        jp.t_jacket = 0.05
        jp.d_tower_base = 6.0
        jp.d_tower_top = 3.87
        jp.t_tower_base = 0.027
        jp.t_tower_top = 0.019

        check_gradient_unit_test(self, jp, display=True)

# TODO: move these to commonse tests - remove RNA content from tower
class TestRNAMass(unittest.TestCase):

    def test1(self):

        rna = RNAMass()
        rna.blades_mass = 15241.323 * 3
        rna.hub_mass = 50421.4
        rna.nac_mass = 221245.8
        rna.hub_cm = [-6.3, 0., 3.15]
        rna.nac_cm = [-0.32, 0., 2.4]
        rna.blades_I = [26375976., 13187988., 13187988., 0., 0., 0.]
        rna.hub_I = [127297.8, 127297.8, 127297.8, 0., 0., 0.]
        rna.nac_I = [9908302.58, 912488.28, 1160903.54, 0., 0., 0.]

        check_gradient_unit_test(self, rna, tol=1e-5)


    def test2(self):

        rna = RNAMass()
        rna.blades_mass = np.random.rand(1)[0]
        rna.hub_mass = np.random.rand(1)[0]
        rna.nac_mass = np.random.rand(1)[0]
        rna.hub_cm = np.random.rand(3)
        rna.nac_cm = np.random.rand(3)
        rna.blades_I = np.random.rand(6)
        rna.hub_I = np.random.rand(6)
        rna.nac_I = np.random.rand(6)

        check_gradient_unit_test(self, rna)



class TestRotorLoads(unittest.TestCase):

    def test1(self):

        loads = RotorLoads()
        loads.F = [123.0, 0.0, 0.0]
        loads.M = [4843.0, 0.0, 0.0]
        loads.r_hub = [2.0, -3.2, 4.5]
        loads.rna_cm = [-3.0, 1.6, -4.0]
        loads.m_RNA = 200.0
        loads.tilt = 13.2
        loads.g = 9.81

        check_gradient_unit_test(self, loads, tol=2e-6)


    def test2(self):

        loads = RotorLoads()
        loads.F = [123.0, 0.0, 0.0]
        loads.M = [4843.0, 0.0, 0.0]
        loads.r_hub = [2.0, -3.2, 4.5]
        loads.rna_cm = [-3.0, 1.6, -4.0]
        loads.m_RNA = 200.0
        loads.tilt = 13.2
        loads.g = 9.81
        loads.downwind = True

        check_gradient_unit_test(self, loads, tol=2e-6)


    def test3(self):

        loads = RotorLoads()
        loads.F = [123.0, 101.0, -50.0]
        loads.M = [4843.0, -2239.0, 1232.0]
        loads.r_hub = [2.0, -3.2, 4.5]
        loads.rna_cm = [-3.0, 1.6, -4.0]
        loads.m_RNA = 200.0
        loads.tilt = 13.2
        loads.g = 9.81

        check_gradient_unit_test(self, loads)


    def test4(self):

        loads = RotorLoads()
        loads.F = [123.0, 101.0, -50.0]
        loads.M = [4843.0, -2239.0, 1232.0]
        loads.r_hub = [2.0, -3.2, 4.5]
        loads.rna_cm = [-3.0, 1.6, -4.0]
        loads.m_RNA = 200.0
        loads.tilt = 13.2
        loads.g = 9.81
        loads.rna_weightM = False

        check_gradient_unit_test(self, loads)




class TestGeometricConstraints(unittest.TestCase):

    def test1(self):

        gc = GeometricConstraints()
        gc.d = [4.0, 3.0, 2.0]
        gc.t = [0.4, 0.23, 0.14]


        check_gradient_unit_test(self, gc)

"""

if __name__ == "__main__":

    # fast = unittest.TestSuite()
    # fast.addTest(TestTowerDiscretization('test3'))
    # unittest.TextTestRunner().run(fast)


    unittest.main()



"""Provides the `ProjectDevelopment` class."""

__author__ = "Jake Nunemaker"
__copyright__ = "Copyright 2020, National Renewable Energy Laboratory"
__maintainer__ = "Jake Nunemaker"
__email__ = "jake.nunemaker@nrel.gov"


from wisdem.orbit.library import extract_library_specs

from .design_phase import DesignPhase


class ProjectDevelopment(DesignPhase):
    """Project Development Class."""

    expected_config = {
        "project_development": {
            "site_auction_duration": "h (optional)",
            "site_auction_price": "USD(optional)",
            "site_assessment_plan_duration": "h (optional)",
            "site_assessment_plan_cost": "USD (optional)",
            "site_assessment_duration": "h (optional)",
            "site_assessment_cost": "USD (optional)",
            "construction_operations_plan_duration": "h (optional)",
            "construction_operations_plan_cost": "USD (optional)",
            "boem_review_duration": "h (optional)",
            "boem_review_cost": "USD (optional)",
            "design_install_plan_duration": "h (optional)",
            "design_install_plan_cost": "USD (optional)",
        }
    }

    output_config = {}

    def __init__(self, config, **kwargs):
        """
        Creates an instance of ProjectDevelopment.

        Parameters
        ----------
        config : dict
        """

        config = self.initialize_library(config, **kwargs)
        self.config = self.validate_config(config)
        self._outputs = {}

        self.defaults = extract_library_specs("defaults", "project")

    def run(self):
        """
        Main run function. Passes ``self.config['project_development']`` to the
        following methods:

        - :py:meth:`.site_auction`
        - :py:meth:`.site_assessment_plan_development`
        - :py:meth:`.site_assessment`
        - :py:meth:`.construction_operations_plan_development`
        - :py:meth:`.boem_review`
        - :py:meth:`.design_install_plan_development`
        """

        dev_specs = self.config.get("project_development", {})

        self.site_auction(**dev_specs)
        self.site_assessment_plan_development(**dev_specs)
        self.site_assessment(**dev_specs)
        self.construction_operations_plan_development(**dev_specs)
        self.boem_review(**dev_specs)
        self.design_install_plan_development(**dev_specs)

    @property
    def design_result(self):
        """
        Returns design results for ProjectDevelopment. This method currently
        returns an empty dictionary as ProjectDevelopment does not produce any
        additional config keys.
        """

        return {}

    @property
    def total_phase_cost(self):
        """Returns total phase cost in $USD."""

        phase_cost = sum([v["cost"] for v in self._outputs.values()])
        return phase_cost

    @property
    def total_phase_time(self):
        """Returns total phase time in hours."""

        phase_time = sum([v["duration"] for v in self._outputs.values()])
        return phase_time

    @property
    def detailed_output(self):
        """Returns detailed phase information."""

        if not self._outputs:
            raise Exception("Has ProjectDevelopment been ran yet?")

        return self._outputs

    def site_auction(self, **development_specs):
        """
        Cost and duration associated with lease area auction.

        Parameters
        ----------
        development_specs : dict
            Dicitonary containing development specifications. Keys:

            - ``site_auction_duration`` - Auction duration in hours.
            - ``site_auction_cost`` - Auction price.
        """

        t = "site_auction_duration"
        c = "site_auction_cost"
        self._outputs["site_auction"] = {
            "duration": development_specs.get(t, self.defaults[t]),
            "cost": development_specs.get(c, self.defaults[c]),
        }

    def site_assessment_plan_development(self, **development_specs):
        """
        Cost and duration associated with developing a site assessment plan.

        Parameters
        ----------
        development_specs : dict
            Dicitonary containing development specifications. Keys:

            - ``site_assessment_plan_duration`` - Site assesment plan
              development duration in hours.
            - ``site_assessment_plan_cost`` - Site assessment plan development
              cost.
        """

        t = "site_assessment_plan_duration"
        c = "site_assessment_plan_cost"
        self._outputs["site_assessment_plan"] = {
            "duration": development_specs.get(t, self.defaults[t]),
            "cost": development_specs.get(c, self.defaults[c]),
        }

    def site_assessment(self, **development_specs):
        """
        Cost and duration for conducting site assessments/surveys and
        obtaining permits.

        Parameters
        ----------
        development_specs : dict
            Dicitonary containing development specifications. Keys:

            - ``site_assessment_duration`` - Site assesment duration in hours.
            - ``site_assessment_cost`` - Site assessment costs.
        """

        t = "site_assessment_duration"
        c = "site_assessment_cost"
        self._outputs["site_assessment"] = {
            "duration": development_specs.get(t, self.defaults[t]),
            "cost": development_specs.get(c, self.defaults[c]),
        }

    def construction_operations_plan_development(self, **development_specs):
        """
        Cost and duration for developing the Construction and Operations Plan
        (COP). Typically occurs in parallel to Site Assessment.

        Parameters
        ----------
        development_specs : dict
            Dicitonary containing development specifications. Keys:

            - ``construction_operations_plan_duration`` - Construction and
              operations plan development duration in hours.
            - ``construction_operations_plan_cost`` - Construction and
              operations plan development cost.
        """

        t = "construction_operations_plan_duration"
        c = "construction_operations_plan_cost"
        self._outputs["construction_operations_plan"] = {
            "duration": development_specs.get(t, self.defaults[t]),
            "cost": development_specs.get(c, self.defaults[c]),
        }

    def boem_review(self, **development_specs):
        """
        Cost and duration for BOEM to review the Construction and Operations
        Plan.

        Parameters
        ----------
        development_specs : dict
            Dicitonary containing development specifications. Keys:

            - ``boem_review_duration`` - BOEM review duration in hours.
            - ``boem_review_cost`` - BOEM review cost. Typically not a cost to
              developers.
        """

        t = "boem_review_duration"
        c = "boem_review_cost"
        self._outputs["boem_review"] = {
            "duration": development_specs.get(t, self.defaults[t]),
            "cost": development_specs.get(c, self.defaults[c]),
        }

    def design_install_plan_development(self, **development_specs):
        """
        Cost and duration for developing the Design and Installation Plan.
        Typically occurs in parallel with BOEM Review.

        Parameters
        ----------
        development_specs : dict
            Dicitonary containing development specifications. Keys:

            - ``design_install_plan_duration`` - Design and installation plan
              development duration in hours.
            - ``design_install_plan_cost`` - Design and installation plan
              development cost.
        """

        t = "design_install_plan_duration"
        c = "design_install_plan_cost"
        self._outputs["design_install_plan"] = {
            "duration": development_specs.get(t, self.defaults[t]),
            "cost": development_specs.get(c, self.defaults[c]),
        }

    @property
    def design_result(self):
        """
        Returns design results for ProjectDevelopment. This method currently
        returns an empty dictionary as ProjectDevelopment does not produce any
        additional config keys.
        """

        return {}



#-------------------------------------------------------------------------------
# Name:        SoilC.py
# Purpose:     This module contains the basic Soil Class
#
# Author:      rdamiani
#
# Created:     26/01/2014 - Based on BuildMPtwr.py SoilC and ReadSoilInfo
# Copyright:   (c) rdamiani 2014
# Licence:     <Apache>
#-------------------------------------------------------------------------------
import warnings
from math import * #this is to allow the input file to have mathematical/trig formulas in their param expressions
import numpy as np
import scipy.interpolate as interpolate
#______________________________________________________________________________#
def ReadSoilInfo(SoilInfoFile):
    """This function creates a soil object, filling main params and then \n
        reads in a file containing class data prepared for Matlab processing.\n
        Thus it spits out an augmented version of the object.
         """
         #Now read soil data
    if isinstance(SoilInfoFile,str) and os.path.isfile(SoilInfoFile): #it means we need to read a file
         soil=SoilC() #This creates the object with some default values
         execfile(SoilInfoFile) #This simply includes the file where I have my input deck also available to matlab
         attr1= (attr for attr in dir(soil) if not attr.startswith('_'))
        #Update MP attributes based on input parameters

    else:
         warnings.warn('SoilInfoFile was not found !')

    return soil

#______________________________________________________________________________#
class SoilC():
            #Soil Class
    def __init__(self,  **kwargs):
        """This function creates a soil class.\n
        """
    #start by setting default values in a dictionary fashion
        Pprms={'zbots':-np.array([3.,5.,7.,15.,30.,50.]), 'gammas':np.array([10000,10000,10000,10000,10000,10000]),\
        'cus':np.array([60000,60000,60000,60000,60000,60000]), 'phis':np.array([36.,33.,26.,37.,35.,37.5]),\
        'delta':25.,'sndflg':True, 'plug':False, 'bwtable':True, 'PenderSwtch':False, 'SoilSF':2., 'qu': 200.e3}
        prms=Pprms.copy()
        prms.update(kwargs)
        for key in kwargs:
            if key not in Pprms:
                setattr(self,key,kwargs[key])  #In case user set something else not included in the Pprms
        for key in Pprms:
            #self.key=prms[key] This does not work instead,beats me
            setattr(self,key,prms[key]) #This takes care of updating values without overburdening the MP class with other stuff which belong to MP

def SubgrReact(soilobj,Lp, sndflg=True, bwtable=True):
        """This function returns the coefficient of subgrade reation ks [N/m3].\n
           For Sands, it comes form the API curves as a function of friction angles.\n
           For Clays, it is an average from Bowles (1968).\n
           Note: This assumes that an average value of ks is to be calculated across all fo the layers.\n
           Es(z)=Es0+ks*z. This is an approximation based on Pender's paper and Matlock and Reese (1956-1975).
           INPUTS:\n
           soilobj      -object of class soil.\n
           Lp           -float, length of pile under ground (embedment length) .\n
           sndflg       -boolean, True for sand, Flase for clay.\n
           bwtable      -boolean, True for below water table (always for offshore), False for above.\n"""
        #
        APIphis=np.array([28.,29.,30.,33.,36.,38.,40.,42.5,45.]) #[deg] friction angles for the API ks table
        minphi=min(APIphis)
        maxphi=max(APIphis)
        APIks=np.array([[5., 12.5, 34.375, 60.9375, 93.75, 121.875, 156.25, 181.25, 221.875], \
                         [0., 12.5, 46.875, 92.1875, 159.375, 212.5, 279.6875, 325, 378.125]]) # [lbf/in3] First row for below water table, 2nd for above- SANDS ONLY
        APIks *=271447.1610  #This converts from lbf/in3 to N/m3

        tks=APIks[int(-bwtable)+1,:]
        deltazs=np.hstack((-soilobj.zbots[0],soilobj.zbots-np.roll(soilobj.zbots,-1)))[:-1]
        if min(soilobj.zbots) <= (-Lp):
            idx=np.nonzero( soilobj.zbots <= (-Lp) )[0][0]#first index of zbots exceeding the z of the pile tip
        else:
            idx=len(deltazs)  #In this case we are assuming the soil is constant below the deepeset level known
        if sndflg:
            #f=interpolate.interp1d(APIphis,tks,kind='quadratic',bounds_error=False)  #function containing the interpolation function
            f=interpolate.UnivariateSpline(APIphis,tks,k=2)  #funct
            #idx2= (soilobj.phis > maxphi) #out of bounds points -only for basic interp1d
            #idx3= (soilobj.phis < minphi) #out of bounds points -only for basic interp1d
            f2=f(soilobj.phis)
            #f2[idx2]=tks[-1]  #replicate the value at teh upper bound for those points exceeding it -only for basic interp1d
            #f2[idx3]=tks[0]   #replicate the value at teh lower bound for those points exceeding it -only for basic interp1d
            ks= ( ((f2*deltazs)[0:idx]).sum()+f2[idx-1]*(Lp+soilobj.zbots[idx-1]) )/Lp   #weigthed average of ks [N/m3]
        else: #For clays there is no direct relationship; Bowles suggests 12,000-48,000 depending on soil capacity
            warnings.warn('Clay Coefficient of Subgrade Reaction to be given as input, watch what is getting calculated and inputted.')
            if soilobj.qu<=200.e3 :
                ks=(soilobj.qu-100.e3)/(200.e3-100.e3)*(24000.e3-12000.e3)+12000.e3   #[N/m3]
            elif 200.e3<soilobj.qu<=800.e3:
                ks=(soilobj.qu-200.e3)/(800.e3-200.e3)*(48000.e3-24000.e3)+24000.e3   #[N/m3]
            elif 800.e3<soilobj.qu:
                ks=48000.e3

        return ks/soilobj.SoilSF

def SoilPileStiffness(ks,Dp,Lp,Ep,Gp,Jxx_p,loadZ=0,nus=0.5,PenderSwtch=False,sndflg=True, H=[],M=[],batter=np.nan,psi=-45.*np.pi/180.):
    """This function returns a 6x6 stiffness matrix relative to mudline, assuming a \n
       coefficient of subgrade reation ks [N/m3] linear with depth below mudline.\n
       It uses either Pender's elastic soil medium approximation or the Matlock and Reese (1960)'s  form.\n
       Es(z)=Es0+ks*z. This is an approximation based on Pender's paper and Matlock and Reese (1956-1975).
       INPUTS:\n
       ks           -float, average soil coefficient of subgrade reaction [N/m3].\n
       Dp           -float, pile OD [m].\n
       Lp           -float, pile embedment length [m].\n
       Ep           -float, pile Young's module, [N/m2].\n
       Gp           -float, pile Shear module, [N/m2].\n
       Jxx_p        -float, pile x-section area moment of inertia, [m4].\n
       loadZ        -float, application point above ground level of H and M.
       nus          -float, Poisson's ratio for soil. Default =0.5 from Pender's 1993.
       PenderSwtch  -boolean, True for Pender's version of flexibility coefficients, False for Matlock and Reese(1960)\n
       sndflg       -boolean, True for sand, Flase for clay.\n
       H            -float, Shear at the top of the pile, positive along x: MANDATORY IF PenderSwtch=True \n
       M            -floar, Moment at the top of the pile: MANDATORY IF PenderSwtch=True \n
       batter       -float, 2D batter in the xz plane for the pile: positive batter means tip is to the left of head and H>0 is pointing to the right \n
       psi          -float, [deg] angle of the pile projection on the x,y plane, for a 4 legged jacket it is -45 deg \n"""

    #General Parameters
    LL=Lp/Dp
    EJxx_p=Ep*Jxx_p

    if PenderSwtch:
        if H or M:
            Es_D=ks*Dp
            K=Ep/Es_D
            La=1.3*Dp*K**(0.222) #active length of pile

            #if Lp>=La:  #long(flexible) pile
            fxH=3.2*K**(-.333)/(Es_D*Dp) #CxF
            fxM=ftH=5.*K**(-.556)/(Es_D*Dp**2)  #CxM=CthtF
            ftM=13.6*K**(-0.778)/(Es_D*Dp**3)   #CthtM
            L_Mmax=0.41*La #Location of maximum moment from ground level
            if sndflg:
                fxH=2.14*K**(-0.29)/(Es_D*Dp) #CxF
                fxM=ftH=3.43*K**(-0.53)/(Es_D*Dp**2)  #CxM=CthtF
                ftM=12.16*K**(-0.77)/(Es_D*Dp**3)   #CthtM


            M=M+loadZ*H #account for eccentricity
            f=M/(Dp*H)  #Eccentricity of the load at the pile head
            a=0.6*f
            b=0.17*f**(-0.3)
            IMH=np.min(np.array([8.,a*K**b]))
            Mmax=IMH*Dp*H
            if Lp<= 0.07*Dp*np.sqrt(Ep/Es_D):  #short rigid pile, in which case Es is also considered a constant
                fxH=0.7*LL**(-.33)/(Es_D*Dp) #CxF
                fxM=ftH=0.4*LL**(-0.88)/(Es_D*Dp**2)  #CxM=CthtF
                ftM=0.6*LL**(-1.67)/(Es_D*Dp**3)   #CthtM
            elif Lp<La:  #intermediate length: use 1.25 the calculated values; correct for fxH, but pushing it for the others
                fxH*=1.25 #CxF
                fxM*=1.25 #CxM
                ftH *=1.25  #CxM=CthtF
                ftM *=1.25   #CthtM
                L_Mmax *=1.25
        else:
            warnings.warn('If using Pender''s Method, H and M must be included')
            sys.exit('!!!ABORT: You must speicfy both H and M when using Pender''s method. Check Soil Inputs, in case use PenderSwtch=False.!!!')

    else:  #Use MAtlock and Reese (1960)
        T=(EJxx_p/ks)**(1./5.) #relative stiffness factor for Es=ks*z type, though it should be Es=Es0+ks*z
        fxH=2.43*T**3/EJxx_p #CxF
        fxM=ftH=1.62*T**2/EJxx_p  #CxM=CthtF
        ftM=1.75*T/EJxx_p   #CthtM

    #Invert the flexibility coefficient matrix
    den=fxH*ftM-fxM**2  #determinant of matrix
    Kmat=1./den * np.array([[ftM, -fxM] ,[-ftM , fxH]])  #This is all in a coordinate system fixed with the pile, with z along its axis
    #Add the axial stiffness now from Pender, assuming linear variation of Es with depth
    E_SL=ks*Lp  #Moudulus at tip of pile
    RR=Ep/E_SL
    b=LL/RR
    Kz=1.8*E_SL*Dp*LL**0.55*RR**(-b)

    #Add torsional stiffness, approximating from Guo and Randolph as flexible pile head stiffness with linear trend of Es.
    Gpeq=32*Gp*Jxx_p/(np.pi*Dp**4) #equivalent G for pile
    Lc=Dp/16. * (2*(1+nus)*Gpeq/(ks*Dp))**(1./3) #critical torsional pile length
    Gsc=ks*Lc/(2*(1+nus)) #soil G at critical length
    Kpsipsi=np.pi/16 * np.sqrt(2) * Dp**3 *np.sqrt(Gpeq/Gsc)

    #Assemble a 6x6 matrix to be returned, with all terms positive, since we do care about abs values not actual direction of forces
    Klocal=np.zeros([6,6]) #Initialize pile head stiffness matrix, this is at the mudline
    Klocal[0,0]=Klocal[1,1]=Kmat[0,0]  #Kx=Ky
    Klocal[0,4]=Klocal[4,0]=Kmat[0,1]  #Kx_thetay=Kthetay_x :force along x due to unit rotation about y
    Klocal[2,2]=Kz
    Klocal[1,3]=Klocal[3,1]=Kmat[0,1]  #Ky_thetax=Kthetax_y :force along y due to unit rotation about x
    Klocal[3,3]=Klocal[4,4]=Kmat[1,1]  #Kthetax_thetax=Ktheta_y_thetay
    Klocal[5,5]=Kpsipsi   #torsional stiffness

    Kglobal=Klocal #initialize for vertical piles

    if np.isfinite(batter) and batter:
        al_bat3D=np.arctan(np.sqrt(2.)/batter)
        cpsi=np.cos(psi)  #cos psi
        spsi=np.sin(psi)  #sin psi
        ca3D=np.cos(al_bat3D)  #cos batter 3d angle
        sa3D=np.sin(al_bat3D)  #sin batter 3d angle

        Cl2g=np.zeros([6,6]) #Initialize Transformtation matrix from local to global
        Cl2g[0,0]=Cl2g[3,3]=cpsi*ca3D
        Cl2g[0,1]=Cl2g[3,4]=-spsi
        Cl2g[0,2]=Cl2g[3,5]=-sa3D*cpsi
        Cl2g[1,0]=Cl2g[4,3]=ca3D*spsi
        Cl2g[1,1]=Cl2g[4,4]=cpsi
        Cl2g[1,2]=Cl2g[4,5]=-spsi*sa3D
        Cl2g[2,0]=Cl2g[5,3]=sa3D
        Cl2g[2,2]=Cl2g[5,5]=ca3D

        Kglobal=np.dot(Cl2g,np.dot(Klocal,Cl2g.T))

    return Kglobal
#______________________________________________________________________________#

if __name__ == '__main__':

    sndflg=True
    soil=SoilC(sndflg=True)
    Lp=36. #embedment length of pile
    Dp=1.5 #OD of pile [m]
    tp=0.06 #thickness of pile [m]
    Ep=2.1e11 #Youngs modulus
    Gp=Ep/(1+0.3)/2. #Shear modulus of pile
    Jxx_p=np.pi/64.*(Dp**4-(Dp-2.*tp)**4)
    loadsz=1  #level abover mudline for shear&moment application
    batter=np.nan
    H=100.e3  #[N] shear
    M=150.e3  #[Nm] moment

    PenderSwtch=True

    print 'Soil z-bottoms [m]:',soil.zbots,' ;\n','Undrained shear strength [N/m^2]:',soil.cus,' ;\n',\
          'Unit weight  [N/m^3]:', soil.gammas,' ;\n','Friction angles [deg]:', soil.phis,' ;\n',\
          'Pile-soil friction angle [deg]:', soil.delta

    ksavg=SubgrReact(soil,Lp,sndflg=sndflg)

    print 'Coefficient of subgrade reaction [MN/m3]=', SubgrReact(soil,Lp,sndflg=sndflg)/1.e6

    Kglobal=SoilPileStiffness(ksavg,Dp,Lp,Ep,Gp,Jxx_p,loadZ=0,PenderSwtch=False,H=H,M=M,batter=batter)

    for ii in range(0,6):
        for jj in range(0,6):
            print('K[{:d},{:d}] = {:10.3e}'.format(ii,jj,Kglobal[ii,jj]))



#-------------------------------------------------------------------------------
# Name:        PlotTower.Py
# Purpose:
#
# Author:      rdamiani
#
# Created:     11/24/2014
# Copyright:   (c) rdamiani 2014
# Licence:     <Apache 2014>
#-------------------------------------------------------------------------------
import numpy as np

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

def main(mytwr,util=False,savefileroot=[]):

    twr_z=mytwr.tower1.z
    twr_D=mytwr.tower1.d

    fig = plt.figure()
    ax = fig.add_subplot(111,aspect='equal')

    plt.plot(-twr_D/2, twr_z)
    plt.plot( twr_D/2, twr_z)
    ax.set_xlim([-60,60]);

#    ax.auto_scale_xyz([min(XYZ1[:,0]),max(XYZ1[:,0])],[min(XYZ1[:,1]),max(XYZ1[:,1])],[min(XYZ1[:,2]),max(XYZ1[:,2])])

    if savefileroot:
        plt.savefig(savefileroot+'_config.png',format='png')
    else:
        plt.show()

    #Plot utilization of Tower if requested
    if util:

        fig2=plt.figure(figsize=(8.0, 5))
        ax2 = fig2.add_subplot(111)
        ax2.set_title('Utilization Ratio');
        #plt.subplot2grid((3, 3), (0, 0), colspan=2, rowspan=3)

        yrange=(np.min(twr_z),np.max(twr_z));
        ax2.set_xlim([0,2]);
        ax2.set_ylim(yrange);

        ax2.plot(mytwr.stress1, twr_z, label='VonMises Util1')
        ax2.plot(mytwr.stress2, twr_z, label='VonMises Util2')
        ax2.plot(mytwr.shellBuckling1, twr_z, label='EUsh Util1')
        ax2.plot(mytwr.shellBuckling2,twr_z, label='EUsh Util2')
        ax2.plot(mytwr.buckling1, twr_z, label='GL Util1')
        ax2.plot(mytwr.buckling2, twr_z, label='GL Util2')
        ax2.grid()
        if any(mytwr.damage):
            ax2.plot(mytwr.damage, twr_z, label='damage')
        ax2.legend(bbox_to_anchor=(1.0, 1.0), loc=1)
        ax2.set_xlabel('Utilization Ratios ')
        ax2.set_ylabel('z from base of tower [m]')

        #Plot tower profile
        ax3=plt.twiny(ax2)# .axes(ax1.get_position(),frameon=False)
        #hh2=plt.axes('Position', pos,'NextPlot','Add','XtickLabel','','Xtick',[],frameon=False);
        ax3.plot(twr_D/2,twr_z);
        ax3.plot(-twr_D/2,twr_z);
        ax3.set_aspect('equal')
        ax3.set_frame_on(False)
        ax3.set_xticklabels('')
        ax3.set_xticks([])
        ax2.set_xlim([0,2]);
        #hh2.set_aspect('equal')
        ##ax2.axis('equal')

        if savefileroot:
            plt.savefig(savefileroot+'_util.png',format='png')
        else:
            plt.show()


if __name__ == '__main__':

    import MyTowerInputs
    mytwr=MyTowerInputs.main()[0]
    mytwr.run()
    main(mytwr,util=True)



import numpy as np
from openmdao.api import Component, Group, ParallelGroup
from airfoilprep import Polar, Airfoil


class AirfoilPreppyPolarExtrapolator(Component):

    def __init__(self, config, sdim, nalpha):

        super(AirfoilPreppyPolarExtrapolator, self).__init__()

        # HAWC2 output requires:
        self.blend_var = config['AirfoilPrep']['blend_var']
        self.naero_coeffs = len(self.blend_var)

        # TODO: predict number of aoa for each airfoil or use same for all afs
        '''
        self.naoa = config['AirfoilPrep']['naoa']
        '''

        self.sdim = sdim
        self.nsec = sdim[1]

        self.nalpha = nalpha
        self.res = config['AirfoilPrep']['res']
        self.nre = len(self.res)

        # airfoil calculation method, e.g. clean, rough, 3D correction method
        self.nmet = len(config['AirfoilPrep']['analysis_methods'])

        self._init_params()
        self._init_unknowns()

        self.useCM = True
        # set standard values
        # maximum drag coefficient
        self.cdmax = 0.0
        # minimum drag coefficient. used to prevent negative values that can
        # sometimes occur with this extrapolation method
        self.cdmin = 0.001
        # number of points to add in each segment of Viterna method
        self.nalpha = 15

        self.plot_polars = False

        for k, w in config['AirfoilPrep'].iteritems():
            try:
                setattr(self, k, w)
            except:
                pass

    def _init_params(self):

        self.add_param('cs_polars', np.zeros(  # nsec, alpha, cl ,cd, cm
            (self.nalpha, 4, self.nsec, self.nre, self.nmet)))
        self.add_param(
            'n_cs_alpha', np.zeros((self.nsec, self.nre, self.nmet)).astype(int))
        self.add_param('cs_polars_tc', np.zeros((self.nsec)))

        self.add_param('AR', 0., desc='aspect ratio = (rotor radius / chord_75% radius)\
            if provided, cdmax is computed from AR')

        self.add_param('rotor_diameter', 0.)
        self.add_param('blade_length', 0.)
        self.add_param('s_st', np.zeros(self.nsec))
        self.add_param('chord_st', np.zeros(self.nsec))

    def _init_unknowns(self):
        # TODO: output as HAWC2/FAST format
        self.add_output('airfoildata:blend_var', np.zeros(self.naero_coeffs))
        '''
        for i in range(self.naero_coeffs):
            self.add_output('airfoildata:aoa%02d' % i, np.zeros(self.naoa))
            self.add_output('airfoildata:cl%02d' % i, np.zeros(self.naoa))
            self.add_output('airfoildata:cd%02d' % i, np.zeros(self.naoa))
            self.add_output('airfoildata:cm%02d' % i, np.zeros(self.naoa))
        '''
        pass

    def solve_nonlinear(self, params, unknowns, resids):

        # determine aspect ratio = (rotor radius / chord_75% radius)\
        #    if provided, cdmax is computed from AR'
        bl = params['blade_length']
        dr = params['rotor_diameter']
        hr = 0.5 * dr - bl
        rotor_radius = 0.5 * dr
        chord = params['chord_st'] * bl
        s = params['s_st']
        r = (s * bl + hr) / rotor_radius
        chord_75 = np.interp(0.75, r, chord)
        AR = rotor_radius / chord_75

        # write aerodyn files
        af_name_base = 'cs_'
        af_name_suffix = '_aerodyn'
        tcs = params['cs_polars_tc']

        n_cs_alpha = params['n_cs_alpha']
        pol = params['cs_polars']
        nmet = 0
        # TODO: blend polars determined with different methods
        for i, tc in enumerate(self.blend_var):
            af_name = af_name_base + \
                '%03d_%04d' % (i, tc * 1000) + af_name_suffix
            re_polars = []
            for nre, re in enumerate(self.res):
                # create polar object
                p = Polar(re,
                          pol[:n_cs_alpha[i, nre, nmet], 0, i, nre, nmet],
                          pol[:n_cs_alpha[i, nre, nmet], 1, i, nre, nmet],
                          pol[:n_cs_alpha[i, nre, nmet], 2, i, nre, nmet],
                          pol[:n_cs_alpha[i, nre, nmet], 3, i, nre, nmet])

                # extrapolate polar
                if tc < self.tc_max:
                    p_extrap = p.extrapolate(self.cdmax,
                                             AR,
                                             self.cdmin,
                                             self.nalpha)
                else:
                    p_extrap = p.extrapolate_as_cylinder()
                p_extrap.useCM = self.useCM
                re_polars.append(p_extrap)

                # TODO: output as HAWC2/FAST format
                # See if HAWC can take several af tables with different Re
                # numbers
                '''
                unknowns['airfoildata:aoa%02d' % i] = p_extrap.alpha
                unknowns['airfoildata:cl%02d' % i] = p_extrap.cl
                unknowns['airfoildata:cd%02d' % i] = p_extrap.cd
                unknowns['airfoildata:cm%02d' % i] = p_extrap.cm
                '''

            # create airfoil object
            af = Airfoil(re_polars)
            af.interpToCommonAlpha()
            af.writeToAerodynFile(af_name + '.dat')

            if self.plot_polars:
                figs = af.plot(single_figure=True)
                titles = ['cl', 'cd', 'cm']
                for (fig, title) in zip(figs, titles):
                    fig.savefig(af_name + '_' + title + '.png', dpi=400)
                    fig.savefig(af_name + '_' + title + '.pdf')

        self._get_unknowns(unknowns)

    def _get_unknowns(self, unknowns):
        unknowns['airfoildata:blend_var'] = self.blend_var



# scigSE.py
# 2019 09 12

import sys
import numpy as np
from math import pi, sin, cos, radians, sqrt, log
from ddse_utils import carterFactor

#--------------------------------

class SCIG(object):
	
    def __init__(self):

        super(SCIG, self).__init__()
        self.debug = False

        #self.bearing_position = bearing_position
        
    #------------------------
    
    def compute(self, rad_ag, len_s, h_s, h_r, machine_rating, n_nom, Gearbox_efficiency, I_0, 
                rho_Fe, rho_Copper, B_symax, shaft_cm, shaft_length, debug=False):
        
        if debug or self.debug:
            sys.stderr.write('AG {:9.2f} LS {:9.2f}  HS {:9.2f}  HR {:9.2f}  MR {:7.1f}  NN {:9.2f}  GE {:9.2f}  I0 {:9.2f}\n'.format(rad_ag, 
                             len_s, h_s, h_r, machine_rating, n_nom, Gearbox_efficiency, I_0))
            sys.stderr.write('rF {:9.2f} rC {:9.2f}  Bs {:9.2f}  SC {:9.2f}  SC {:9.2f}  SC {:9.2f}  SL {:9.2f}\n'.format(rho_Fe, 
                             rho_Copper, B_symax, shaft_cm[0], shaft_cm[1], shaft_cm[2], shaft_length))
        
        # Assign values to universal constants
        g1    = 9.81              # m/s^2 acceleration due to gravity
        sigma = 21.5e3            # shear stress (psi) what material?
        mu_0  = pi*4e-7           # permeability of free space in m * kg / (s**2 * A**2)
        cofi  = 0.9               # power factor
        h_w   = 0.005             # wedge height
        m     = 3                 # Number of phases
        resist_Cu = 1.8e-8 * 1.4  # Copper resistivity
        
        #Assign values to design constants
        b_so      = 0.004                     # Stator slot opening width
        b_ro      = 0.004                     # Rotor  slot opening width
        q1        = 6                         # Stator slots per pole per phase
        q2        = 4                         # Rotor  slots per pole per phase
        b_s_tau_s = 0.45                      # Stator Slot width/Slot pitch ratio
        b_r_tau_r = 0.45                      # Rotor Slot width/Slot pitch ratio
        y_tau_p   = 12./15                    # Coil span/pole pitch
        
        p         = 3                         # number of pole pairs
        freq      = 60                        # frequency in Hz
        k_fillr   = 0.7                       # Rotor  slot fill factor
        k_fills   = None                      # Stator slot fill factor - to be assigned later
        P_Fe0h    = 4                         # specific hysteresis losses W / kg @ 1.5 T @50 Hz
        P_Fe0e    = 1                         # specific eddy losses W / kg @ 1.5 T @50 Hz
        
        S_N       = -0.002                    # Slip
        n_1       = n_nom / (1 - S_N)         # actual rotor speed (rpm)
        
        # Calculating winding factor 
        
        k_y1      = sin(pi/2 * y_tau_p)       # winding chording factor
        k_q1      = sin(pi/6) / (q1 * sin(pi/(6 * q1))) # zone factor
        k_wd      = k_y1 * k_q1               # winding factor                
        
        # Calculating air gap length
        ag_dia     = 2 * rad_ag               # air gap diameter
        ag_len     = (0.1 + 0.012 * machine_rating**(1./3)) * 0.001 # air gap length in m
        K_rad      = len_s / ag_dia             # Aspect ratio
        K_rad_LL   = 0.5                      # lower limit on aspect ratio
        K_rad_UL   = 1.5                      # upper limit on aspect ratio
        rad_r        = rad_ag - ag_len        # rotor radius
                                              
        tau_p      = pi * ag_dia / (2 * p)    # pole pitch
        S          = 2 * p * q1 * m           # Stator slots                                             
        N_slots_pp = S / (m * p * 2)          # Number of stator slots per pole per phase
        tau_s      = tau_p / (m * q1)         # Stator slot pitch
                                              
        b_s        = b_s_tau_s * tau_s        # Stator slot width
        b_t        = tau_s - b_s              # Stator tooth width
                                              
        Q_r        = 2 * p * m * q2           # Rotor slots
        tau_r      = pi * (ag_dia - 2 * ag_len) / Q_r # Rotor slot pitch
        b_r        = b_r_tau_r * tau_r        # Rotor slot width
        b_tr       = tau_r - b_r              # Rotor tooth width
        tau_r_min  = pi * (ag_dia - 2 * (ag_len + h_r)) / Q_r
        b_trmin    = tau_r_min - b_r_tau_r * tau_r_min # minumum rotor tooth width
        
        
        # Calculating equivalent slot openings
        mu_rs       = 0.005
        mu_rr       = 0.005
        W_s         = (b_s / mu_rs) * 1e-3  # Stator, in m
        W_r         = (b_r / mu_rr) * 1e-3  # Rotor,  in m
        
        Slot_aspect_ratio1 = h_s / b_s      # Stator slot aspect ratio
        Slot_aspect_ratio2 = h_r / b_r      # Rotor slot aspect ratio
	                             
        # Calculating Carter factor for stator,rotor and effective air gap length
        '''
        gamma_s = (2 * W_s / ag_len)**2 / (5 + 2 * W_s / ag_len)
        K_Cs    = tau_s / (tau_s - ag_len * gamma_s * 0.5) # page 3-13 Boldea Induction machines Chapter 3
        gamma_r = (2 * W_r / ag_len)**2 / (5 + 2 * W_r / ag_len)
        K_Cr    = tau_r / (tau_r - ag_len * gamma_r * 0.5) # page 3-13 Boldea Induction machines Chapter 3
        '''
        
        K_Cs    = carterFactor(ag_len, W_s, tau_s)
        K_Cr    = carterFactor(ag_len, W_r, tau_r)
        K_C     = K_Cs * K_Cr
        g_eff   = K_C * ag_len
                  
        om_m    = 2 * pi * n_nom / 60     # mechanical frequency
        om_e    = p * om_m                # electrical frequency
        f       = n_nom * p / 60          # generator output freq
        K_s     = 0.3                     # saturation factor for Iron
        n_c     = 2                       # number of conductors per coil
        a1      = 2                       # number of parallel paths
        
        # Calculating stator winding turns
        N_s    = np.round(2 * p * N_slots_pp * n_c / a1)
        
        # Calculating Peak flux densities
        B_g1    = mu_0 * 3 * N_s * I_0 * sqrt(2) * k_y1 * k_q1 / (pi * p * g_eff * (1 + K_s))
        B_g     = B_g1 * K_C
        B_rymax = B_symax
        
        # calculating back iron thickness
        h_ys = B_g * tau_p / (B_symax * pi)
        h_yr = h_ys
        
        d_se    = ag_dia + 2 * (h_ys + h_s + h_w)  # stator outer diameter
        D_ratio = d_se / ag_dia                    # Diameter ratio
        
        # limits for Diameter ratio depending on pole pair
        if (2 * p == 2):
            D_ratio_LL = 1.65
            D_ratio_UL = 1.69
        elif (2 * p == 4):
            D_ratio_LL = 1.46
            D_ratio_UL = 1.49
        elif (2 * p == 6):
            D_ratio_LL = 1.37
            D_ratio_UL = 1.4
        elif (2 * p == 8):
            D_ratio_LL = 1.27
            D_ratio_UL = 1.3
        else:
            D_ratio_LL = 1.2
            D_ratio_UL = 1.24
            
        # Stator slot fill factor
        if ag_dia > 2:
            k_fills = 0.65
        else:
            k_fills = 0.4
        
        # Stator winding length and cross-section
        l_fs      = 2 * (0.015 + y_tau_p * tau_p / 2 / cos(radians(40))) + pi * h_s # end connection
        l_Cus     = 2 * N_s * (l_fs + len_s) / a1                                     # shortpitch
        A_s       = b_s *        (h_s - h_w)                                        # Slot area
        A_scalc   = b_s * 1000 * (h_s - h_w) * 1000                                 # Conductor cross-section (mm^2)
        A_Cus     = A_s     * q1 * p * k_fills / N_s                                # Conductor cross-section (m^2)        
        A_Cuscalc = A_scalc * q1 * p * k_fills / N_s
        
        # Stator winding resistance
        R_s          = l_Cus * resist_Cu / A_Cus
        
        # Calculating no-load voltage
        om_s        = n_nom * 2 * pi / 60                     # rated angular frequency            
        P_e         = machine_rating / (1 - S_N)              # Electrical power
        E_p         = om_s * N_s * k_wd * rad_ag * len_s * B_g1 * sqrt(2)
        
        S_GN        = (1.0 - S_N) * machine_rating # same as P_e?
        T_e         = p * S_GN / (2 * pi * freq * (1 - S_N))
        I_srated    = machine_rating / (3 * E_p * cofi)
        
        #Rotor design
        diff        = h_r - h_w
        A_bar       = b_r * diff                                # bar cross section
        Beta_skin   = sqrt(pi * mu_0 * freq / 2 / resist_Cu)    # coefficient for skin effect correction
        k_rm        = Beta_skin * h_r                           # coefficient for skin effect correction
        J_b         = 6e+06                                     # Bar current density
        K_i         = 0.864
        I_b         = 2 * m * N_s * k_wd * I_srated / Q_r       # bar current
        
        # Calculating bar resistance
        
        R_rb        = resist_Cu * k_rm * len_s / A_bar
        I_er        = I_b / (2 * sin(pi * p / Q_r))             # End ring current
        J_er        = 0.8 * J_b                                 # End ring current density
        A_er        = I_er / J_er                               # End ring cross-section
        b           = h_r                                       # End ring dimension
        a           = A_er / b                                  # End ring dimension
        D_er        = (rad_ag * 2 - 2 * ag_len) - 0.003         # End ring diameter
        l_er        = pi * (D_er - b) / Q_r                     # End ring segment length
        if debug:
            sys.stderr.write('l_er {:.4f} A_er {:.4f} D_er {:.4f}\n'.format(l_er[0], A_er[0], D_er[0]))
        
        # Calculating end ring resistance
        R_re = resist_Cu * l_er / (2 * A_er * (sin(pi * p / Q_r))**2)
        
        # Calculating equivalent rotor resistance
        if debug:
            sys.stderr.write('R_rb {:.3e} R_re {:.3e} k_wd {:.4f} N_s {} Q_r {}\n'.format(R_rb, R_re, k_wd, N_s, Q_r))
        R_R = (R_rb + R_re) * 4 * m * (k_wd * N_s)**2 / Q_r
        
        # Calculating Rotor and Stator teeth flux density
        B_trmax = B_g * tau_r / b_trmin
        B_tsmax = B_g * tau_s / b_t
        
        # Calculating Equivalent core lengths
        l_r  = len_s + 4     * ag_len   # for axial cooling
        l_se = len_s + (2/3) * ag_len
        K_fe = 0.95              # Iron factor
        L_e = l_se * K_fe        # radial cooling
        
        # Calculating leakage inductance in  stator
        if debug:
            sys.stderr.write('b_s {:.3e} b_so {:.3e}\n'.format(b_s[0], b_so[0]))
        L_ssigmas  = 2 * mu_0 * len_s * N_s**2 / p / q1 * ((h_s - h_w) / (3 * b_s) + h_w / b_so)                        # slot        leakage inductance
        L_ssigmaew = 2 * mu_0 * len_s * N_s**2 / p / q1 * 0.34 * q1 * (l_fs - 0.64 * tau_p * y_tau_p) / len_s           # end winding leakage inductance
        L_ssigmag  = 2 * mu_0 * len_s * N_s**2 / p / q1 * (5 * (ag_len * K_C / b_so) / (5 + 4 * (ag_len * K_C / b_so))) # tooth tip   leakage inductance
        L_s        = L_ssigmas + L_ssigmaew + L_ssigmag                                                                 # stator      leakage inductance
        L_sm       = 6 * mu_0 * len_s * tau_p * (k_wd * N_s)**2 / (pi**2 * p * g_eff * (1 + K_s))
        
        # Calculating leakage inductance in  rotor
        lambda_ei = 2.3 * D_er / (4 * Q_r * len_s * (sin(pi * p / Q_r)**2)) * log(4.7 * ag_dia / (a + 2 * b))
        lambda_b  = h_r / (3 * b_r) + h_w / b_ro
        L_i       = pi * ag_dia / Q_r
        
        L_rsl = mu_0 * len_s * ((h_r - h_w) / (3 * b_r) + h_w / b_ro)       # slot        leakage inductance
        L_rel = mu_0 * (len_s * lambda_b + 2 * lambda_ei * L_i)             # end winding leakage inductance
        L_rtl = mu_0 * len_s * (0.9 * tau_r * 0.09 / g_eff)                 # tooth tip   leakage inductance
        L_rsigma = (L_rsl + L_rtl + L_rel) * 4 * m * (k_wd * N_s)**2 / Q_r  # rotor       leakage inductance
        
        # Calculating rotor current
        if debug:
            sys.stderr.write('S_N {} P_e {:.1f} m {} R_R {:.4f} = {:.1f}\n'.format(S_N, P_e, m, R_R, -S_N * P_e / m / R_R))
        I_r = sqrt( -S_N * P_e / m / R_R)
        
        I_sm = E_p / (2 * pi * freq * L_sm)
        # Calculating stator currents and specific current loading
        I_s = sqrt((I_r**2 + I_sm**2))
        
        A_1 = 2 * m * N_s * I_s / (pi * 2 * rad_ag)
        
        # Calculating masses of the electromagnetically active materials
        
        V_Cuss = m * l_Cus * A_Cus                                                         # Volume of copper in stator
        V_Cusr = Q_r * len_s * A_bar + pi * (D_er * A_er - A_er * b)                         # Volume of copper in rotor
        V_Fest = len_s * pi * ((rad_ag + h_s)**2 - rad_ag**2) - 2 * m * q1 * p * b_s * h_s * len_s   # Volume of iron in stator teeth
        V_Fesy = len_s * pi * ((rad_ag + h_s + h_ys)**2 - (rad_ag + h_s)**2)                       # Volume of iron in stator yoke
        rad_r = rad_ag - ag_len                                                               # rotor radius
        
        V_Fert = pi * len_s * (rad_r**2 - (rad_r - h_r)**2) - 2 * m * q2 * p * b_r * h_r * len_s # Volume of iron in rotor teeth
        V_Fery = pi * len_s * ((rad_r - h_r)**2 - (rad_r - h_r - h_yr)**2)                     # Volume of iron in rotor yoke
        Copper = (V_Cuss + V_Cusr) * rho_Copper            # Mass of Copper
        M_Fest = V_Fest * rho_Fe                           # Mass of stator teeth
        M_Fesy = V_Fesy * rho_Fe                           # Mass of stator yoke
        M_Fert = V_Fert * rho_Fe                           # Mass of rotor tooth
        M_Fery = V_Fery * rho_Fe                           # Mass of rotor yoke
        Iron = M_Fest + M_Fesy + M_Fert + M_Fery
        
        Active_mass = Copper + Iron
        L_tot = len_s
        Structural_mass = 0.0001 * Active_mass**2 + 0.8841 * Active_mass - 132.5
        Mass = Active_mass + Structural_mass
        
        # Calculating Losses and efficiency
        
        # 1. Copper losses
        
        K_R = 1.2                        # skin effect correction coefficient
        P_Cuss = m * I_s**2 * R_s * K_R  # Copper loss - stator
        P_Cusr = m * I_r**2 * R_R        # Copper loss - rotor
        P_Cusnom = P_Cuss + P_Cusr       # Copper loss - total
        
        # Iron Losses ( from Hysteresis and eddy currents)          
        P_Hyys = M_Fesy * (B_symax / 1.5)**2 * (P_Fe0h * om_e / (2 * pi * 60))                 # Hysteresis losses in stator yoke
        P_Ftys = M_Fesy * (B_symax / 1.5)**2 * (P_Fe0e * (om_e / (2 * pi * 60))**2)            # Eddy       losses in stator yoke
        P_Hyd  = M_Fest * (B_tsmax / 1.5)**2 * (P_Fe0h * om_e / (2 * pi * 60))                 # Hysteresis losses in stator tooth
        P_Ftd  = M_Fest * (B_tsmax / 1.5)**2 * (P_Fe0e * (om_e / (2 * pi * 60))**2)            # Eddy       losses in stator tooth
        P_Hyyr = M_Fery * (B_rymax / 1.5)**2 * (P_Fe0h * abs(S_N) * om_e / (2 * pi * 60))      # Hysteresis losses in rotor yoke
        P_Ftyr = M_Fery * (B_rymax / 1.5)**2 * (P_Fe0e * (abs(S_N) * om_e / (2 * pi * 60))**2) # Eddy       losses in rotor yoke
        P_Hydr = M_Fert * (B_trmax / 1.5)**2 * (P_Fe0h * abs(S_N) * om_e / (2 * pi * 60))      # Hysteresis losses in rotor tooth
        P_Ftdr = M_Fert * (B_trmax / 1.5)**2 * (P_Fe0e * (abs(S_N) * om_e / (2 * pi * 60))**2) # Eddy       losses in rotor tooth
        
        # Calculating Additional losses
        P_add = 0.5 * machine_rating / 100
        P_Fesnom = P_Hyys + P_Ftys + P_Hyd + P_Ftd + P_Hyyr + P_Ftyr + P_Hydr + P_Ftdr
        Losses = P_Cusnom + P_Fesnom + P_add
        gen_eff = (P_e - Losses) * 100 / P_e
        Overall_eff = gen_eff * Gearbox_efficiency
        
        # Calculating current densities in the stator and rotor
        J_s = I_s / A_Cuscalc
        J_r = I_r / A_bar / 1e6
        
        # Calculating Tangential stress constraints
        TC1 = T_e / (2 * pi * sigma)
        TC2 = rad_ag**2 * len_s
        
        # Calculating mass moments of inertia and center of mass
        
        I = np.zeros(3)
        r_out = d_se * 0.5
        I[0]   = 0.50 * Mass * r_out**2
        I[1]   = 0.25 * Mass * r_out**2 + Mass * len_s**2 / 12
        I[2]   = I[1]
        cm = np.zeros(3)
        cm[0]  = shaft_cm[0] + shaft_length/2. + len_s/2.
        cm[1]  = shaft_cm[1]
        cm[2]  = shaft_cm[2]

        return B_tsmax, B_trmax, B_rymax, B_g, B_g1, q1, N_s, S, h_ys, b_s, b_t, D_ratio, D_ratio_UL, \
            D_ratio_LL, A_Cuscalc, Slot_aspect_ratio1, h_yr, tau_p, p, Q_r, b_r, b_trmin, b_tr, \
            rad_r, S_N, A_bar, Slot_aspect_ratio2, E_p, f, I_s, A_1, J_s, J_r, R_s, R_R, L_s, \
            L_sm, Mass, K_rad, K_rad_UL, K_rad_LL, Losses, gen_eff, Copper, Iron, Structural_mass, \
            TC1, TC2, Overall_eff, cm, I                  

'''        
        outputs = {}
        outputs['B_tsmax']            = B_tsmax
        outputs['B_trmax']            = B_trmax
        outputs['B_rymax']            = B_rymax
        outputs['B_g']                = B_g
        outputs['B_g1']               = B_g1
        outputs['q1']                 = q1
        outputs['N_s']                = N_s
        outputs['S']                  = S
        outputs['h_ys']               = h_ys
        outputs['b_s']                = b_s
        outputs['b_t']                = b_t
        outputs['D_ratio']            = D_ratio
        outputs['D_ratio_UL']         = D_ratio_UL
        outputs['D_ratio_LL']         = D_ratio_LL
        outputs['A_Cuscalc']          = A_Cuscalc
        outputs['Slot_aspect_ratio1'] = Slot_aspect_ratio1
        outputs['h_yr']               = h_yr
        outputs['tau_p']              = tau_p
        outputs['p']                  = p
        outputs['Q_r']                = Q_r
        outputs['b_r']                = b_r
        outputs['b_trmin']            = b_trmin
        outputs['b_tr']               = b_tr
        outputs['rad_r']              = rad_r
        outputs['S_N']                = S_N
        outputs['A_bar']              = A_bar
        outputs['Slot_aspect_ratio2'] = Slot_aspect_ratio2
        outputs['E_p']                = E_p
        outputs['f']                  = f
        outputs['I_s']                = I_s
        outputs['A_1']                = A_1
        outputs['J_s']                = J_s
        outputs['J_r']                = J_r
        outputs['R_s']                = R_s
        outputs['R_R']                = R_R
        outputs['L_s']                = L_s
        outputs['L_sm']               = L_sm
        outputs['Mass']               = Mass
        outputs['K_rad']              = K_rad
        outputs['K_rad_UL']           = K_rad_UL
        outputs['K_rad_LL']           = K_rad_LL
        outputs['Losses']             = Losses
        outputs['gen_eff']            = gen_eff
        outputs['Copper']             = Copper
        outputs['Iron']               = Iron
        outputs['Structural_mass']    = Structural_mass
        outputs['TC1']                = TC1
        outputs['TC2']                = TC2
        outputs['Overall_eff']        = Overall_eff
        outputs['cm']                 = cm
        outputs['I']                  = I
'''
 
#%%-----------------

        
if __name__ == '__main__':

    scig = SCIG()
    
    r_s                = 0.55    # meter
                                 
    len_s                = 1.30    # meter
    h_s                = 0.090   # meter
    h_r                = 0.050   # meter
    machine_rating     = 5000000.0
    n_nom              = 1200.0
    Gearbox_efficiency = 0.955        
    I_0                = 140     # Ampere
    rho_Fe             = 7700.0  # Steel density Kg/m3
    rho_Copper         = 8900.0  # copper density Kg/m3       
    B_symax            = 1.4     # Tesla
    shaft_cm           = np.array([0.0, 0.0, 0.0])
    shaft_length       = 2.0
        
    rad_ag = r_s
        
    outputs = scig.compute(rad_ag, len_s, h_s, h_r, machine_rating, n_nom, Gearbox_efficiency, I_0, 
                rho_Fe, rho_Copper, B_symax, shaft_cm, shaft_length)


      



import cProfile
import pstats

import floatingse.floating as ff
import floatingse.semiInstance as se

#cProfile.run('ff.semiExample()','profout')
cProfile.run('se.psqp_optimal()','profout')
p = pstats.Stats('profout')
n = 40
# Clean up filenames for the report
p.strip_dirs()

p.sort_stats('cumulative').print_stats(n)
p.sort_stats('time').print_stats(n)





__author__ = "Jake Nunemaker"
__copyright__ = "Copyright 2020, National Renewable Energy Laboratory"
__maintainer__ = "Jake Nunemaker"
__email__ = "jake.nunemaker@nrel.gov"


from copy import deepcopy

from wisdem.orbit import ProjectManager

config = {
    "wtiv": "test_wtiv",
    "site": {"depth": 20, "distance": 20, "mean_windspeed": 9},
    "plant": {"num_turbines": 20},
    "turbine": {
        "hub_height": 130,
        "rotor_diameter": 154,
        "rated_windspeed": 11,
    },
    "port": {"num_cranes": 1, "monthly_rate": 2e6},
    "monopile": {
        "type": "Monopile",
        "length": 60,
        "diameter": 8,
        "deck_space": 0,
        "mass": 600,
    },
    "transition_piece": {
        "type": "Transition Piece",
        "deck_space": 0,
        "mass": 500,
    },
    "monopile_design": {},
    "design_phases": ["MonopileDesign"],
    "install_phases": ["MonopileInstallation"],
}


def test_monopile_definition():

    test_config = deepcopy(config)
    _ = test_config.pop("transition_piece")

    project = ProjectManager(test_config)
    project.run_project()

    for key, value in config["monopile"].items():
        if key == "type":
            continue

        assert project.config["monopile"][key] == value

    for key, value in config["transition_piece"].items():
        if key == "type":
            continue

        assert project.config["transition_piece"][key] != value


def test_transition_piece_definition():

    test_config = deepcopy(config)
    _ = test_config.pop("monopile")

    project = ProjectManager(test_config)
    project.run_project()

    for key, value in config["monopile"].items():
        if key == "type":
            continue

        assert project.config["monopile"][key] != value

    for key, value in config["transition_piece"].items():
        if key == "type":
            continue

        assert project.config["transition_piece"][key] == value


def test_mono_and_tp_definition():

    test_config = deepcopy(config)

    project = ProjectManager(test_config)
    project.run_project()

    for key, value in config["monopile"].items():
        if key == "type":
            continue

        assert project.config["monopile"][key] == value

    for key, value in config["transition_piece"].items():
        if key == "type":
            continue

        assert project.config["transition_piece"][key] == value
